/**
 * @file  Channel4ModuleHardware.cpp
 * @brief Implementierung der Klasse CChannel4ModuleHardware.
 */
//////////////////////////////////////////////////////////////////////

//#include "StdAfx.h"
#include "Channel4ModuleHardware.h"
#include <stdio.h>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
void CChannel4ModuleHardware::InitClass(short channelID, short deviceID)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::InitClass [ %i, %i ]\n", __FILE__, __LINE__, channelID, deviceID);

}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
CChannel4ModuleHardware::~CChannel4ModuleHardware() 
{
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleHardware::GetErrorModule(short module, short error, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetErrorModule [ %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, error, text);

	// Ist IV_MODULE_HARDWARE ?
	if (module == IV_MODULE_HARDWARE) {

		switch(error) {
		case 1: sprintf(text, "( %i ): Das geforderte Modul wird nicht untersttzt.", error); break;
		case 2: sprintf(text, "( %i ): Die geforderte Funktion wird nicht untersttzt.", error); break;
		case 3: sprintf(text, "( %i ): Es wurden bei der Funktion zu wenige Parameter angegeben.", error); break;
		case 4: sprintf(text, "( %i ): Es wurden bei der Funktion zu viele Parameter angegeben.", error); break;
		default: sprintf(text, "( %i ): Nicht eingetragener Fehler!", error);
		}

		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4Protocol::GetErrorModule(module, error, text);
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleHardware::GetErrorFunction(short module, short function, short error, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetErrorFunction [ %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, function, error, text);

	if (module == IV_MODULE_HARDWARE) {

		switch(function) {
		case IV_FUNCTION_PING:
			sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error);
			break;
		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}
		
		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4Protocol::GetErrorFunction(module, function, error, text);
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleHardware::Request(short deviceIDRemote)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::Request [ %i ]\n", __FILE__, __LINE__, deviceIDRemote);

	int err;
	unsigned char type;
	short module, function;
	int count = 0;

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Paket senden
	if ((err = TransmitPacket(GetDeviceIDRemote(), IV_PACKET_REQUEST, IV_MODULE_HARDWARE, IV_FUNCTION_PING, m_data, 0)) != IV_ERROR_SUCCESS) {

		return err;
	}

	// Paket abholen
	if ((err = ReceivePacket(GetDeviceID(), type, module, function, m_data, count, 1)) != IV_ERROR_SUCCESS) {

		return err;
	}

	if ((*(unsigned short*)&m_transmit[4] + sizeof(unsigned short)) != *(unsigned short*)&m_receive[4]) {

		// Ungleiche Pingantwort in Datenlaenge
		Trace("%s:(%i):ERROR(1): CChannel4ModuleHardware::Request (Ungleiche Datenlaenge)\n", __FILE__, __LINE__);
	}

	return IV_ERROR_SUCCESS;
}

//#########################################################
//###################### HARDWARE #########################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleHardware::CallFunction(short module, short function, unsigned char *data, int &count)
{
	if (module == IV_MODULE_HARDWARE) {
		switch(function) {
		case IV_FUNCTION_PING: return IV_ERROR_SUCCESS;

		// Setzen/Ermitteln des Geraetetyp
//		case IV_FUNCTION_SET_TYPE_DEVICE: return CallSetTypeDevice(data, count);
		case IV_FUNCTION_GET_TYPE_DEVICE: return CallGetTypeDevice(data, count); 

		// Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
//		case IV_FUNCTION_SET_TYPE_BOARD: return CallSetTypeBoard(data, count);
		case IV_FUNCTION_GET_TYPE_BOARD: return CallGetTypeBoard(data, count);

		// Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
//		case IV_FUNCTION_SET_COMPONENT_RS485_0: return CallSetComponentRS485_0(data, count);
		case IV_FUNCTION_GET_COMPONENT_RS485_0: return CallGetComponentRS485_0(data, count);

		// Setzen/Ermitteln der Boardkomponente: TCP/IP
//		case IV_FUNCTION_SET_COMPONENT_TCPIP: return CallSetComponentTCPIP(data, count);
		case IV_FUNCTION_GET_COMPONENT_TCPIP: return CallGetComponentTCPIP(data, count);

		// Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
//		case IV_FUNCTION_SET_TYPE_MODEM_MC: return CallSetTypeModemMC(data, count);
		case IV_FUNCTION_GET_TYPE_MODEM_MC: return CallGetTypeModemMC(data, count);

		// Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
//		case IV_FUNCTION_SET_COMPONENT_RS485_1: return CallSetComponentRS485_1(data, count);
		case IV_FUNCTION_GET_COMPONENT_RS485_1: return CallGetComponentRS485_1(data, count);

		// Setzen/Ermitteln des Transpondertyps
//		case IV_FUNCTION_SET_TYPE_TRANSPONDER: return CallSetTypeTransponder(data, count);
		case IV_FUNCTION_GET_TYPE_TRANSPONDER: return CallGetTypeTransponder(data, count);

		// Setzen/Ermitteln des Displaytyp
//		case IV_FUNCTION_SET_TYPE_DISPLAY: return CallSetTypeDisplay(data, count);
		case IV_FUNCTION_GET_TYPE_DISPLAY: return CallGetTypeDisplay(data, count);

		// Setzen/Ermitteln des Tastaturtyps
//		case IV_FUNCTION_SET_TYPE_KEYPAD: return CallSetTypeKeypad(data, count);
		case IV_FUNCTION_GET_TYPE_KEYPAD: return CallGetTypeKeypad(data, count);

		// Setzen/Ermitteln der Multimediakarte
//		case IV_FUNCTION_SET_TYPE_MMC: return CallSetTypeMMC(data, count);
		case IV_FUNCTION_GET_TYPE_MMC: return CallGetTypeMMC(data, count);

		// Setzen/Ermitteln des GPS - Typs
//		case IV_FUNCTION_SET_TYPE_GPS: return CallSetTypeGPS(data, count);
		case IV_FUNCTION_GET_TYPE_GPS: return CallGetTypeGPS(data, count);

		// Setzen/Ermitteln der USB - Komponente
//		case IV_FUNCTION_SET_COMPONENT_USB: return CallSetComponentUSB(data, count);
		case IV_FUNCTION_GET_COMPONENT_USB: return CallGetComponentUSB(data, count);

		// Setzen/Ermitteln der Hardwareconfigurationsnummer
//		case IV_FUNCTION_SET_HW_CONFIG: return CallSetHWConfig(data, count);
		case IV_FUNCTION_GET_HW_CONFIG: return CallGetHWConfig(data, count);

		// Setzen/Ermitteln des Funkmodultyps
//		case IV_FUNCTION_SET_TYPE_RADIO: return CallSetTypeRadio(data, count);
		case IV_FUNCTION_GET_TYPE_RADIO: return CallGetTypeRadio(data, count);

		// Setzen/Ermitteln des Funkuhrmodultyps
//		case IV_FUNCTION_SET_TYPE_DCF77: return CallSetTypeDCF77(data, count);
		case IV_FUNCTION_GET_TYPE_DCF77: return CallGetTypeDCF77(data, count);

		// Setzen/Ermitteln der analogen Eingaenge - Komponente
//		case IV_FUNCTION_SET_COMPONENT_IN_ANALOG: return CallSetComponentInAnalog(data, count);
		case IV_FUNCTION_GET_COMPONENT_IN_ANALOG: return CallGetComponentInAnalog(data, count);

		// Setzen/Ermitteln der digitalen Eingaenge - Komponente
//		case IV_FUNCTION_SET_COMPONENT_IN_DIGITAL: return CallSetComponentInDigital(data, count);
		case IV_FUNCTION_GET_COMPONENT_IN_DIGITAL: return CallGetComponentInDigital(data, count);

		// Setzen/Ermitteln der digitalen Ausgaenge - Komponente
//		case IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL: return CallSetComponentOutDigital(data, count);
		case IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL: return CallGetComponentOutDigital(data, count);

		// Setzen/Ermitteln des Fingerprint + Typs
//		case IV_FUNCTION_SET_TYPE_FINGERPRINT: return CallSetTypeFingerprint(data, count);
		case IV_FUNCTION_GET_TYPE_FINGERPRINT: return CallGetTypeFingerprint(data, count);

		// Funktion nicht definiert
		default: return 0x0002; 
		}
	}

	return CChannel4Protocol::CallFunction(module, function, data, count);
}

// ##################################################################
/**
 * @brief Senden eines Anfragepakets an das gewuenschte Geraet.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleHardware::Ping(short deviceIDRemote)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::Ping [ %i ]\n", __FILE__, __LINE__, deviceIDRemote);

	int count = 0;

	// Umgebung zuruecksetzen
	ResetError();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Request(deviceIDRemote)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_PING);
		return m_result;
	}
	
	return m_result;
}

// ##################################################################
/**
 * @brief Ermitteln des Geraetetyps.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Geraetetyp
 *             0 = PZE - Master IV
 *             1 = Zutritts - Master IV
 *             2 = BDE - Master IV
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeDevice(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeDevice [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeDevice [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermitteln des Geraetetyps.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Geraetetyp
 *             0 = PZE - Master IV
 *             1 = Zutritts - Master IV
 *             2 = BDE - Master IV
 *           ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeDevice(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeDevice [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_TYPE_DEVICE_PZE                 0 // PZE-MasterIV
#define IV_VALUE_TYPE_DEVICE_ZK                  1 // ZK-MasterIV
#define IV_VALUE_TYPE_DEVICE_BDE                 2 // BDE-MasterIV
#define IV_VALUE_TYPE_DEVICE_MOBIL               3 // Mobil-MasterIV
#define IV_VALUE_TYPE_DEVICE_MDE                 4 // MDE-BoxIV
#define IV_VALUE_TYPE_DEVICE_AE                  5 // AE-MasterIV
#define IV_VALUE_TYPE_DEVICE_TIMEBOY             6 // Timeboy-MasterIV
*/
	data[0] = IV_VALUE_TYPE_DEVICE_PZE;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt die vorliegende Board-Revisionsnummer.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Revisionsnummer in der Form (HiNibble Majornummer).(LoNibble Minornummer)
 *             z. B. 0x14 -> Major 1, Minor (Redesign Nr.) 4
 */
int CChannel4ModuleHardware::GetTypeBoard(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeBoard [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeBoard [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt die vorliegende Board-Revisionsnummer.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Revisionsnummer in der Form (HiNibble Majornummer).(LoNibble Minornummer)
 *             z. B. 0x14 -> Major 1, Minor (Redesign Nr.) 4
 */
int CChannel4ModuleHardware::CallGetTypeBoard(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeBoard [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_TYPE_BOARD_1_0                  0x10
#define IV_VALUE_TYPE_BOARD_1_1                  0x11
#define IV_VALUE_TYPE_BOARD_1_2                  0x12
#define IV_VALUE_TYPE_BOARD_1_3                  0x13
#define IV_VALUE_TYPE_BOARD_1_4                  0x14
#define IV_VALUE_TYPE_BOARD_2_0                  0x20
#define IV_VALUE_TYPE_BOARD_2_1                  0x21
*/
	data[0] = IV_VALUE_TYPE_BOARD_2_0;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentRS485_0(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetComponentRS485_0 [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetComponentRS485_0 [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::CallGetComponentRS485_0(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetComponentRS485_0 [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	data[0] = 0;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentTCPIP(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetComponentTCPIP [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_TCPIP, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_TCPIP);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetComponentTCPIP [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::CallGetComponentTCPIP(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetComponentTCPIP [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	data[0] = 1;
	count = sizeof(unsigned char);

	return m_result;
}
// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *              0 = Keines
 *              1 = MC35i
 *              2 = MC55
 *            ... = ...
 */
int CChannel4ModuleHardware::GetTypeModemMC(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeModemMC [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeModemMC [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *              0 = Keines
 *              1 = MC35i
 *              2 = MC55
 *            ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeModemMC(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeModemMC [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_TYPE_MODEM_MC_NONE              0 // Kein GSM/GPRS
#define IV_VALUE_TYPE_MODEM_MC_39                1 // Siemens MC35i/MC39
#define IV_VALUE_TYPE_MODEM_MC_55                2 // Siemens MC55
*/
	data[0] = IV_VALUE_TYPE_MODEM_MC_NONE;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentRS485_1(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetComponentRS485_1 [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetComponentRS485_1 [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::CallGetComponentRS485_1(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetComponentRS485_1 [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	data[0] = 0;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keines
 *             1 = Lehner kleines
 *             2 = Lehner grosses
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeDisplay(short deviceIDRemote, unsigned char &value, int checked)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeDisplay [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	// Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
	// - Funktionen TypeTransponder liefern <-> Displaywert
	// - Funktionen TypeDisplay liefern <-> Keypadwert
	// - Funktionen TypeKeypad liefern <-> Transponderwert
	//
	// Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeDisplay [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keines
 *             1 = Lehner kleines
 *             2 = Lehner grosses
 *           ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeDisplay(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeDisplay [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_TYPE_DISPLAY_NONE               0 // Kein Display
#define IV_VALUE_TYPE_DISPLAY_SMALL              1 // Lehner Dabitros (320 x 240)
#define IV_VALUE_TYPE_DISPLAY_BIG                2 // Lehner Dabitros (320 x 240)
*/
	data[0] = IV_VALUE_TYPE_DISPLAY_SMALL;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = PZE - Master IV
 *             2 = BDE - Master IV
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeKeypad(short deviceIDRemote, unsigned char &value, int checked)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeKeypad [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	// Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
	// - Funktionen TypeTransponder liefern <-> Displaywert
	// - Funktionen TypeDisplay liefern <-> Keypadwert
	// - Funktionen TypeKeypad liefern <-> Transponderwert
	//
	// Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeKeypad [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = PZE - Master IV
 *             2 = BDE - Master IV
 *           ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeKeypad(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeKeypad [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_TYPE_KEYPAD_NONE                0 // Keine Tastatur
#define IV_VALUE_TYPE_KEYPAD_9                   1 // 9 Tasten Tastatur (PZE-MasterIV usw.)
#define IV_VALUE_TYPE_KEYPAD_58                  2 // 56+2 Tasten Tastatur (BDE-MasterIV usw.)
#define IV_VALUE_TYPE_KEYPAD_29                  3 // 28+1 Tasten Tastatur (AE-MasterIV usw.)
*/
	data[0] = IV_VALUE_TYPE_KEYPAD_9;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
* @param value Ermittelte Variante.
 *             0 = Keiner
 *             1 = TSR30
 *             2 = TSR32
 *             3 = PHG PRIMO 100
 *             4 = Cavitec Mifare
 *             5 = Dallas iButton
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeTransponder(short deviceIDRemote, unsigned char &value, int checked)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeTransponder [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	// Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
	// - Funktionen TypeTransponder liefern <-> Displaywert
	// - Funktionen TypeDisplay liefern <-> Keypadwert
	// - Funktionen TypeKeypad liefern <-> Transponderwert
	//
	// Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeTransponder [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
* @param value Ermittelte Variante.
 *             0 = Keiner
 *             1 = TSR30
 *             2 = TSR32
 *             3 = PHG PRIMO 100
 *             4 = Cavitec Mifare
 *             5 = Dallas iButton
 *           ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeTransponder(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeTransponder [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_TYPE_TRANSPONDER_NONE           0 // Kein Transponderleser
#define IV_VALUE_TYPE_TRANSPONDER_TSR30          1 // GIS TSR30
#define IV_VALUE_TYPE_TRANSPONDER_TSR32          2 // GIS TSR32
#define IV_VALUE_TYPE_TRANSPONDER_PRIMO100       3 // PHG Primo 100
#define IV_VALUE_TYPE_TRANSPONDER_MIFARE         4 // Cavitec Mifare
#define IV_VALUE_TYPE_TRANSPONDER_IBUTTON        5 // Dallas iButton
#define IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS    6 // SimonsVoss SmartRelais
*/
	data[0] = IV_VALUE_TYPE_TRANSPONDER_TSR32;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = Fest integriert
 *             2 = Herausnehmbar
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeMMC(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeMMC [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeMMC [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = Fest integriert
 *             2 = Herausnehmbar
 *           ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeMMC(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeMMC [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_TYPE_MMC_NONE                   0 // Keine MMC eingebaut
#define IV_VALUE_TYPE_MMC_EXTENSION              1 // Als Speichererweiterung
#define IV_VALUE_TYPE_MMC_DISK                   2 // Als Wechseldatentraeger
*/
	data[0] = IV_VALUE_TYPE_MMC_NONE;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = Bluegiga (Bluetooth)
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeRadio(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeRadio [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeRadio [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = Bluegiga (Bluetooth)
 *           ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeRadio(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeRadio [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_TYPE_RADIO_NONE                 0 // Kein Funk
#define IV_VALUE_TYPE_RADIO_WRAPTHOR             1 // Bluegiga Wrap Thor
*/
	data[0] = IV_VALUE_TYPE_RADIO_NONE;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *              0 = Kein
 *              1 = Furuno GN-80
 *            ... = ...
 */
int CChannel4ModuleHardware::GetTypeGPS(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeGPS [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeGPS [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *              0 = Kein
 *              1 = Furuno GN-80
 *            ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeGPS(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeGPS [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);
/*
#define IV_VALUE_TYPE_GPS_NONE                   0 // Kein GPS
#define IV_VALUE_TYPE_GPS_GN80                   1 // Furuno GN-80
*/
	data[0] = IV_VALUE_TYPE_GPS_NONE;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentUSB(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetComponentUSB [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetComponentUSB [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::CallGetComponentUSB(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetComponentUSB [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	data[0] = 0;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = HKW
 *             4 = Nicht zu verwenden, da auf 0 gemappt.
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeDCF77(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeDCF77 [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeDCF77 [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	// Wert wurde von FWV 04.01.01 fuer die Baudrate von Bluetooth verwendet und
	// steht nach initialisierung auf dem Wert 4. Dieser Wert, wird als Typ ignoriert.
	value = (m_data[0] != 0xFF && m_data[0] != 4) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = HKW
 *           ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeDCF77(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeDCF77 [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	data[0] = IV_VALUE_TYPE_DCF77_NONE;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}


// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentInAnalog(short deviceIDRemote, unsigned long &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetComponentInAnalog [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned long)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetComponentInAnalog [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (*(unsigned long*)m_data != 0xFFFFFFFF) ? *(unsigned long*)m_data : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::CallGetComponentInAnalog(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetComponentInAnalog [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	*(unsigned long*)data = 0xFFFFFFFF;
	count = sizeof(unsigned long);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentInDigital(short deviceIDRemote, unsigned long &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetComponentInDigital [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned long)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetComponentInDigital [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (*(unsigned long*)m_data != 0xFFFFFFFF) ? *(unsigned long*)m_data : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::CallGetComponentInDigital(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetComponentInDigital [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	*(unsigned long*)data = 0xFFFFFFFF;
	count = sizeof(unsigned long);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetComponentOutDigital(short deviceIDRemote, unsigned long &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetComponentOutDigital [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned long)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetComponentOutDigital [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (*(unsigned long*)m_data != 0xFFFFFFFF) ? *(unsigned long*)m_data : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Komponente enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::CallGetComponentOutDigital(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetComponentOutDigital [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	*(unsigned long*)data = 0xFFFFFFFF;
	count = sizeof(unsigned long);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Option enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Option enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetOptionIOModule(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetOptionIOModule [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetOptionIOModule [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = m_data[0];

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Option enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Option enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetOptionComServer(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetOptionComServer [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetOptionComServer [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = m_data[0];

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt ob die bezeichnete Option enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert ob Option enthalten = 1 / nicht enthalten = 0
 */
int CChannel4ModuleHardware::GetOptionTBAdaption(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetOptionTBAdaption [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetOptionTBAdaption [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = m_data[0];

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = Idencom (BioKey 3000)
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeFingerprint(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeFingerprint [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeFingerprint [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Ermittelte Variante.
 *             0 = Keine
 *             1 = Idencom (BioKey 3000)
 *           ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeFingerprint(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeFingerprint [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	data[0] = IV_VALUE_TYPE_FINGERPRINT_NONE;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Bitfeld der verwendbaren Varianten.
 *            -1 = Undefiniert
 *            Bit 1 = GIS
 *            Bit 2 = Protect
 *            Bit 3 = PHG
 *           ... = ...
 */
int CChannel4ModuleHardware::GetTypeExternalReader(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetTypeExternalReader [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetTypeExternalReader [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// BUGFIX
	// Eine Firmware die die Serien TS und Protect untersttzt setzt als untersttzte Serien
	// TS und Protect obwohl es so sein sollte, dass die angesprochene Serie eingetragen wird,
	// so dass der Wert der Hardwareinfo eindeutig ist.

	// Wert uebernehmen
	value = m_data[0];

	// Werden beide Flags vorgefunden wird von TS alleine ausgegangen.
	if ((value != 0xFF) && (value & IV_VALUE_TYPE_EXTERNAL_READER_GIS) && (value & IV_VALUE_TYPE_EXTERNAL_READER_PROTECT)) {

		value = IV_VALUE_TYPE_EXTERNAL_READER_GIS;
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Bitfeld der verwendbaren Varianten.
 *            -1 = Undefiniert
 *            Bit 1 = GIS
 *            Bit 2 = Protect
 *            Bit 3 = PHG
 *           ... = ...
 */
int CChannel4ModuleHardware::CallGetTypeExternalReader(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetTypeExternalReader [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	data[0] = IV_VALUE_TYPE_EXTERNAL_READER_UNDEFINED;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Ermittelt die "Version" der Hardwareconfiguration.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert fuer Version der Hardwareconfiguration.
 */
int CChannel4ModuleHardware::GetHWConfig(short deviceIDRemote, unsigned char &value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::GetHWConfig [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG);
		return m_result;
	}

	// Antwortparameter pruefen
	if (count != sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleHardware::GetHWConfig [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, count);
		value = 0;
		return m_result;
	}

	// Wert uebernehmen
	value = (m_data[0] != 0xFF) ? m_data[0] : 0;

	return m_result;
}

// ##################################################################
/**
 * @brief Ermittelt die "Version" der Hardwareconfiguration.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 * @param value Wert fuer Version der Hardwareconfiguration.
 */
int CChannel4ModuleHardware::CallGetHWConfig(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleHardware::CallGetHWConfig [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	data[0] = 1;
	count = sizeof(unsigned char);

	return IV_ERROR_SUCCESS;
}


#ifndef DATAFOX_STUDIO_IV

// Funktions-Stumps

int CChannel4ModuleHardware::PingRandom(short deviceIDRemote)                                 { return -IV_ERROR_FUNCTION_UNAVAILABLE; }

int CChannel4ModuleHardware::SetTypeDevice(short deviceIDRemote, unsigned char value)         { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeBoard(short deviceIDRemote, unsigned char value)          { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeModemMC(short deviceIDRemote, unsigned char value)        { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeDisplay(short deviceIDRemote, unsigned char value, int checked)     { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeKeypad(short deviceIDRemote, unsigned char value, int checked)      { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeTransponder(short deviceIDRemote, unsigned char value, int checked) { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeMMC(short deviceIDRemote, unsigned char value)            { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeRadio(short deviceIDRemote, unsigned char value)          { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeGPS(short deviceIDRemote, unsigned char value)            { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeDCF77(short deviceIDRemote, unsigned char value)          { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeFingerprint(short deviceIDRemote, unsigned char value)    { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetTypeExternalReader(short deviceIDRemote, unsigned char value) { return -IV_ERROR_FUNCTION_UNAVAILABLE; }

int CChannel4ModuleHardware::SetComponentRS485_0(short deviceIDRemote, unsigned char value)   { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetComponentTCPIP(short deviceIDRemote, unsigned char value)     { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetComponentRS485_1(short deviceIDRemote, unsigned char value)   { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetComponentUSB(short deviceIDRemote, unsigned char value)       { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetComponentInAnalog(short deviceIDRemote, unsigned long value)  { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetComponentInDigital(short deviceIDRemote, unsigned long value) { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetComponentOutDigital(short deviceIDRemote, unsigned long value){ return -IV_ERROR_FUNCTION_UNAVAILABLE; }

int CChannel4ModuleHardware::SetOptionIOModule(short deviceIDRemote, unsigned char value)     { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetOptionComServer(short deviceIDRemote, unsigned char value)    { return -IV_ERROR_FUNCTION_UNAVAILABLE; }
int CChannel4ModuleHardware::SetOptionTBAdaption(short deviceIDRemote, unsigned char value)   { return -IV_ERROR_FUNCTION_UNAVAILABLE; }

int CChannel4ModuleHardware::SetHWConfig(short deviceIDRemote, unsigned char value)           { return -IV_ERROR_FUNCTION_UNAVAILABLE; }

#endif

