#ifndef _CHANNEL4_MODULE_SYSTEM_H_
#define _CHANNEL4_MODULE_SYSTEM_H_

// Basisklasse einbinden
#include "Channel4ModuleHardware.h"

// MODULE_SYSTEM( 1 )
#define IV_MODULE_SYSTEM                         0x0001

// Fehler:
// 1 -> Modul unbekannt
// 2 -> Funktion unbekannt
// 3 -> Zu wenige Parameter
// 4 -> Zu viele Parameter
// 5 -> Zu wenig Daten
// 6 -> Zu viele Daten

// Ermitteln der Firmwareversion
#define IV_FUNCTION_GET_VERSION                  0x0000
// Format: XX.XX.XX??????????????

// Setzen/Ermitteln von: SerialNum
#define IV_FUNCTION_SET_SERIAL                   0x0001
#define IV_FUNCTION_GET_SERIAL                   0x0002
	
// Fehlernummer:
// 1 = Pruefung der Seriennummer - Verschluesselung schlug fehl.

// Setzen/Ermitteln von: DateTime (YYYY-MM-DD HH:MM:SS\0)
#define IV_FUNCTION_SET_RTC                      0x0003
#define IV_FUNCTION_GET_RTC                      0x0004


// Setzen/Ermitteln von: Variable (Systemvariablen GPRS) 
#define IV_FUNCTION_SET_SYSTEM_VARIABLE          0x0005
#define IV_FUNCTION_GET_SYSTEM_VARIABLE          0x0006

// Der Name einer Systemvariablen setzt sich auf folgende Art zusammen:
// MAJOR_NAME.MINOR_NAME
// Fr GPRS ist der Majorname MODEM_MC35i.
// Fehlernummer:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = Lnge des Variablenwertes ist zu lang.


// Setzen/Ermitteln von: Relaiszustand 
#define IV_FUNCTION_SET_RELAY                    0x0007
#define IV_FUNCTION_GET_RELAY                    0x0008

// Die Namen sind: INTERN, TMR (Trmodule), IOM (IOModule)
// Die Busnummer ist die Busnummer bei 0 beginnend. (Bei INTERN eigentlich immer 0)
// Die Relaisnummer ist die Relaisnummer bei 0 beginnend.
// Die Zeit ist eine Angabe in Sekunden (0 bedeutet eine Dauerschaltung in den gewhlten Status)
// Fehlernummer:
// 1 = Name undefiniert
// 2 = Busnummer undefiniert
// 3 = Relaisnummer undefiniert
// 4 = Status undefiniert  (0, 1) (Knnte irgendwann um andere Varianten erweitert werden)

// Schreiben/Lesen von: Externen Flash
#define IV_FUNCTION_SET_FLASH                    0x0009
#define IV_FUNCTION_GET_FLASH                    0x000A

// Schreiben/Lesen von: EEPROM
#define IV_FUNCTION_SET_EEPROM                   0x000B
#define IV_FUNCTION_GET_EEPROM                   0x000C

// Schreiben/Lesen von: Program-Flash schreiben/lesen
#define IV_FUNCTION_SET_PROGRAM                  0x000D // (wird nur aufgenommen)
#define IV_FUNCTION_GET_PROGRAM                  0x000E

// Starten/Beenden von: Segmentuebertragung (inklusive aktiver Antwort)
#define IV_FUNCTION_BEGIN_SET_SEGMENT                0x000F
#define IV_FUNCTION_BEGIN_SET_SEGMENT_OK             0x0010
#define IV_FUNCTION_END_SET_SEGMENT                  0x0011
// Im Bestaetigungspaket fuer Befehlerhalt ist der CRC = 0 und 
// im Bestaetigungspaket fuer CRC pruefung ist der CRC = dem bei SYSTEM_START gesendeten.
// Ist der CRC falsch, gibt es ein Fehlerpaket mit Fehlernummer 6.

// Die folgenden IV_VALUE_SEGMENT_.. gelten fuer _BEGIN_SEGMENT und _END_SEGMENT
#define IV_VALUE_SEGMENT_UPDATE         0 // Es werden Daten gesetzt die nachdem Uebernommen werden.
#define IV_VALUE_SEGMENT_UPDATE_BG      1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define IV_VALUE_SEGMENT_FIRMWARE       2
#define IV_VALUE_SEGMENT_FONTS          3
#define IV_VALUE_SEGMENT_GRAFICS        4
#define IV_VALUE_SEGMENT_SETUP          5
#define IV_VALUE_SEGMENT_LIST           6
#define IV_VALUE_SEGMENT_SETUP_EXTENDED 7
#define IV_VALUE_SEGMENT_LIST_ACCESS    8 // Listen fuer ZK II werden uebertragen, dadurch wird die Zutrittskontrolle blockiert.
#define IV_VALUE_SEGMENT_LIST_TIMEBOY   9 // Liste fuer Timeboy uebertragen
// 0 -> Global fuer Uebernahmerahmen von Datenbundel
// 1 -> Firmware
// 2 -> Fonts
// 3 -> Grafik
// 4 -> Setup

// Fehler:
// 1 -> Unbekannter SS-Type
// 2 -> Zu viele Updatesegmente (7 vordefiniert)
// 3 -> Angeforderter Speicherplatz steht nicht zur Verfgung
// 4 -> Kein Stopp vor Start
// 5 -> Verschachtelungsfehler von Stopp und Start. {[]()} -> OK, aber {[( -> Fehler
// 6 -> CRC Berechnung schlug fehl.

#define IV_FUNCTION_END_SET_SEGMENT_OK               0x0012
// Wird vom Geraet aufgrund eines SYSTEM_START -> SYSTEM_END mit nachfolgender Berechnung vom CRC gesendet.

// Ermitteln von: Bootloaderversion
#define IV_FUNCTION_GET_VERSION_BOOTLOADER       0x0013
// Format: XX.XX.XX??????????????

// Durchfuehren eines Resets der Speicherbereiche
#define IV_FUNCTION_SYSTEM_RESET                 0x0014

// Resetmodus:
// Geraet in einen Zustand versetzen, wie nach dem Bootloader einspielen.
#define IV_VALUE_RESET_ALL             0
#define IV_VALUE_RESET_RESTART         1

// Fehler:
// 1 -> Resetmodus unbekannt.


// Ins Bios wechseln
#define IV_FUNCTION_ENTER_BIOS			         0x0015
// Bioswechsel erfolgt
#define IV_FUNCTION_ENTERED_BIOS		         0x0016

// Bios verlassen (aenderungen uebernehmen
#define IV_FUNCTION_EXIT_BIOS			         0x0017
// Biosverlassen erfolgt
#define IV_FUNCTION_EXITED_BIOS			         0x0018

// Setzt/liest das zu verwendende Interface
#define IV_FUNCTION_SET_INTERFACE                0x0019
// 0 -> RS232
// 1 -> TCP/IP
// 2 -> GSM
// 3 -> GPRS
// 4 -> RS485
#define IV_FUNCTION_GET_INTERFACE                0x001A

#define IV_VALUE_INTERFACE_RS232 0
#define IV_VALUE_INTERFACE_TCPIP 1
#define IV_VALUE_INTERFACE_GSM   2
#define IV_VALUE_INTERFACE_GPRS  3
#define IV_VALUE_INTERFACE_RS485 4


// Setzt/liest die zu verwendende BaudRate
#define IV_FUNCTION_SET_BAUD                     0x001B
#define IV_FUNCTION_GET_BAUD                     0x001C
// Setzt/liest die zu verwendende DeviceID
#define IV_FUNCTION_SET_DEVICE_ID                0x001D
#define IV_FUNCTION_GET_DEVICE_ID                0x001E

// Befehl der beim Geraetestart den Bootloader des ZK - Master IV wiederherstellt.
#define IV_FUNCTION_ENTER_BOOTLOADER             0x001F

// In Monitor Mode von XPort wechselt
#define IV_FUNCTION_ENTER_XPORT_MONITOR          0x0020
// Antwort in Monitor Mode gewechselt
#define IV_FUNCTION_ENTER_XPORT_MONITOR_OK       0x0021

// Monitorbefehl an XPort und Antwort erhalten.
#define IV_FUNCTION_CMD_XPORT                    0x0022

// In Monitor Mode von XPort wechselt
#define IV_FUNCTION_EXIT_XPORT_MONITOR           0x0023
// Antwort in Monitor Mode gewechselt
#define IV_FUNCTION_EXIT_XPORT_MONITOR_OK        0x0024


#define IV_FUNCTION_BEGIN_GET_SEGMENT            0x0025
// [in]-> [UINT16(SS_...)]
// <- 16bit 
#define IV_FUNCTION_BEGIN_GET_SEGMENT_OK         0x0026
// [out]-> 16bit Typ des Speichers
// [out]-> 32bit Adresse fuer Lesen
// [out]-> 32bit Groesse des Blocks
// [out]-> 32bit CRC des Blocks

#define IV_FUNCTION_END_GET_SEGMENT              0x0027
// -> [UINT16(SS_...)]
// <- [UINT16(SS_...)]
// Im Bestaetigungspaket fuer Befehlerhalt ist der CRC = 0 und 
// im Bestaetigungspaket fuer CRC pruefung ist der CRC = dem bei SYSTEM_START gesendeten.
// Ist der CRC falsch, gibt es ein Fehlerpaket mit Fehlernummer 6.

// Die folgenden IV_VALUE_SEGMENT_.. gelten fuer _BEGIN_GET_SEGMENT und _END_GET_SEGMENT
#define IV_VALUE_SEGMENT_GET_UPDATE         0 // Es werden Daten gelesen ohne Bedienstop.
#define IV_VALUE_SEGMENT_GET_UPDATE_BG      1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define IV_VALUE_SEGMENT_GET_SETUP          5
#define IV_VALUE_SEGMENT_GET_SETUP_EXTENDED 7

// Fehler:
// 1 -> Unbekannter SS-Type
// 2 -> Zu viele Updatesegmente (7 vordefiniert)
// 3 -> Angeforderter Speicherplatz steht nicht zur Verfgung
// 4 -> Kein Stopp vor Start
// 5 -> Verschachtelungsfehler von Stopp und Start. {[]()} -> OK, aber {[( -> Fehler
// 6 -> CRC Berechnung schlug fehl.

#define IV_FUNCTION_END_GET_SEGMENT_OK           0x0028
// Wird vom Geraet aufgrund eines SYSTEM_START -> SYSTEM_END mit nachfolgender Berechnung vom CRC gesendet.

#define IV_FUNCTION_SYSTEM_RESET_OK              0x0029

// Transparentmodus aktivieren/deaktivieren.
#define IV_FUNCTION_OPEN_COM_SERVER_MODE         0x002A
#define IV_FUNCTION_CLOSE_COM_SERVER_MODE        0x002B

// Abfrage von Information zu einem Modultyp
#define IV_FUNCTION_GET_INFO                     0x002C
// param1: ACCESS.MODULE
// param2: bei ACCESS.MODULE UCHAR(BusNr) UCHAR(0=TM,1=L)

// Antwort in Com-Server-Mode gewechselt
#define IV_FUNCTION_OPEN_COM_SERVER_MODE_OK      0x002D

// Aktivieren/Deaktivieren und Abfragen der Benachrichtigungen
#define IV_FUNCTION_SET_NOTIFY                   0x002E
// param1: Typ der Benachrichtigung
// param2: Zu setztender Zustand der Benachrichtigung
#define IV_FUNCTION_GET_NOTIFY                   0x002F
// param1: Typ der Benachrichtigung
// Werte siehe IV_MODULE_APPLICATION.IV_FUNCTION_NOTIFY

#define IV_FUNCTION_GET_SOFTWARE_INFO            0x0030

// Lesen einer Seite des Flash (Bereich: 0 - 4095, Seite hat 528Byte)
#define IV_FUNCTION_GET_FLASH_PAGE               0x0031

class CChannel4ModuleSystem : public CChannel4ModuleHardware
{
public:
	/// Konstruktor fuer Initialisierung der Klasse.
	CChannel4ModuleSystem()  : CChannel4ModuleHardware(IV_DEFAULT_CHANNELID, IV_DEFAULT_DEVICEID) 
	{ InitClass(); }
	CChannel4ModuleSystem(short channelID, short deviceID) : CChannel4ModuleHardware(channelID, deviceID) 
	{ InitClass(channelID, deviceID); }
	virtual void InitClass(short channelID = IV_DEFAULT_CHANNELID, short deviceID = IV_DEFAULT_DEVICEID);
	/// Destruktor
	virtual ~CChannel4ModuleSystem();

public:
	// Gibt Modulfehler zurueck.
	virtual void GetErrorModule(short module, short error, char *text);
	// Gibt Funktionsfehlertext zurueck.
	virtual void GetErrorFunction(short module, short function, short error, char *text);

	// MODULE_HARDWARE
	// BUG:
	// Im Bootloader 00.04.00.00 && 00.04.00.01 liefert die Funktion
	// - TypeTransponder den Wert fuer das Display
	// - TypeDisplay den Wert fuer das Keypad
	// - TypeKeypad den Wert fuer den Transponder
	int SetTypeDisplay(short deviceIDRemote, unsigned char value, int checked = FALSE);
	int GetTypeDisplay(short deviceIDRemote, unsigned char &value, int checked = FALSE);
	int SetTypeKeypad(short deviceIDRemote, unsigned char value, int checked = FALSE);
	int GetTypeKeypad(short deviceIDRemote, unsigned char &value, int checked = FALSE);
	int SetTypeTransponder(short deviceIDRemote, unsigned char value, int checked = FALSE);
	int GetTypeTransponder(short deviceIDRemote, unsigned char &value, int checked = FALSE);

	// Gesamte Hardwareinfo lesen
	int GetHardwareInfo(short deviceIDRemote, TDFHardwareInfo &hwInfo);
	// Gesamte Hardwareinfo in String umsetzen.
	int GetInfos(TDFHardwareInfo hwInfo, char *info);

	
	// MODULE_SYSTEM

	// Systemreset; Speicherbereiche loeschen
	int SystemReset(short deviceIDRemote, unsigned short modus);

	// Firmwareversion lesen
	int GetVersion(short deviceIDRemote, char *version);
	int GetVersionBootloader(short deviceIDRemote, char *version);

	// Seriennummer lesen
	int SetSerial(short deviceIDRemote, unsigned long serial);
	int GetSerial(short deviceIDRemote, unsigned long &serial);

	// Datum/Uhrzeit setzen/lesen
	int SetRTC(short deviceIDRemote, char *time);
	int GetRTC(short deviceIDRemote, char *time);

	// Systemvariable setzen/lesen
	int SetSystemVariable(short deviceIDRemote, char *name, char *value);
	int GetSystemVariable(short deviceIDRemote, char *name, char *value);

	// Relais setzen/lesen
	int SetRelay(short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time);
	int GetRelay(short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time);

	// Datensegmentuebertragung beginnen/enden
	int BeginSetSegment(short deviceIDRemote, unsigned short art, unsigned long &address, unsigned long size, unsigned long crc);
	int EndSetSegment(short deviceIDRemote, unsigned short art, unsigned long crc);

	// Datensegmentuebertragung beginnen/enden
	int BeginGetSegment(short deviceIDRemote, unsigned short art, unsigned long &address, unsigned long &size, unsigned long &crc);
	int EndGetSegment(short deviceIDRemote, unsigned short art);

	// Flash setzen/lesen (die ersten 16kByte Bootloader koennen nur gelesen werden)
	int SetFlash(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);
	int GetFlash(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);
	int GetFlashPage(short deviceIDRemote, unsigned short pageNum, unsigned char *buf, int &count);

	// EEPROM setzen/lesen
	int SetEEPROM(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);
	int GetEEPROM(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);

	// PROGRAM setzen/(lesen unterbunden)
	int SetProgram(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);
	int GetProgram(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);

	// Systembios aufrufen/verlassen
	int EnterBios(short deviceIDRemote);
	int ExitBios(short deviceIDRemote);

	// Interface setzen/lesen (RS232, TCPIP...)
	int SetInterface(short deviceIDRemote, unsigned char type);
	int GetInterface(short deviceIDRemote, unsigned char &type);
	
	// Baud-Rate setzen/lesen
	int SetBaud(short deviceIDRemote, unsigned long br);
	int GetBaud(short deviceIDRemote, unsigned long &br);

	// DeviceID setzen/lesen
	int SetDeviceIDBios(short deviceIDRemote, unsigned short id);
	int GetDeviceIDBios(short deviceIDRemote, unsigned short &id);

	// Bootloader aktivieren
	int EnterBootloader(short deviceIDRemote);

	// Monitor Mode vom XPort
	int EnterXPortMonitor(short deviceIDRemote, char *data);
	int CommandXPort(short deviceIDRemote, char *buf, int &count);
	int ExitXPortMonitor(short deviceIDRemote);

	// Transparentsmodus fuer COM
	int OpenComServerMode(short deviceIDRemote, char *comm, unsigned long baudRate, unsigned char byteSize, unsigned char parity, unsigned char stopBits, unsigned short flags, unsigned short timeoutToClose);
	int CloseComServerMode(short deviceIDRemote);

	// Informationen zu einem Modultyp abfragen
	int GetInfo(short deviceIDRemote, char *name, char *param, char *info, int &infoLength);

	// Aktivieren/Deaktivieren und Abfragen der Benachrichtigungen
	int SetNotify(short deviceIDRemote, unsigned char type, unsigned char state);
	int GetNotify(short deviceIDRemote, unsigned char type, unsigned char &state);

	// Softwareinformation der Anwendungsfirmware abfragen
	int GetSoftwareInfo(short deviceIDRemote, unsigned char *swi, int &infoLength);

	//#################################
	// Call - Routinen

	// Ausfuehren von Befehlen
	virtual int CallFunction(short module, short function, unsigned char *data, int &count);

	// Firmwareversion lesen
	int CallGetVersion(unsigned char *data, int &count);
	int CallGetVersionBootloader(unsigned char *data, int &count);

	// Seriennummer lesen
	int CallGetSerial(unsigned char *data, int &count);

	// Datum/Uhrzeit setzen/lesen
	int CallSetRTC(unsigned char *data, int &count);
	int CallGetRTC(unsigned char *data, int &count);

	// Systemvariable setzen/lesen
	int CallSetSystemVariable(unsigned char *data, int &count);
	int CallGetSystemVariable(unsigned char *data, int &count);

	// Relais setzen/lesen
	int CallSetRelay(unsigned char *data, int &count);
	int CallGetRelay(unsigned char *data, int &count);

	// Datensegmentuebertragung beginnen/enden
	int CallBeginSetSegment(unsigned char *data, int &count);
	int CallBeginSetSegmentOK(unsigned char *data, int &count);
	int CallEndSetSegment(unsigned char *data, int &count);
	int CallEndSetSegmentOK(unsigned char *data, int &count);

	// Datensegmentuebertragung beginnen/enden
	int CallBeginGetSegment(unsigned char *data, int &count);
	int CallBeginGetSegmentOK(unsigned char *data, int &count);
	int CallEndGetSegment(unsigned char *data, int &count);
	int CallEndGetSegmentOK(unsigned char *data, int &count);

	// Flash setzen/lesen (die ersten 16kByte Bootloader koennen nur gelesen werden)
	int CallSetFlash(unsigned char *data, int &count);
	int CallGetFlash(unsigned char *data, int &count);

	// EEPROM setzen/lesen
	int CallSetEEPROM(unsigned char *data, int &count);
	int CallGetEEPROM(unsigned char *data, int &count);

	// PROGRAM setzen/(lesen unterbunden)
//	int CallSetProgram(unsigned char *data, int &count);
//	int CallGetProgram(unsigned char *data, int &count);

	// Systembios aufrufen/verlassen
	int CallEnterBios(unsigned char *data, int &count);
	int CallExitBios(unsigned char *data, int &count);

	// Interface setzen/lesen (RS232, TCPIP...)
	int CallSetInterface(unsigned char *data, int &count);
	int CallGetInterface(unsigned char *data, int &count);
	
	// Baud-Rate setzen/lesen
	int CallSetBaud(unsigned char *data, int &count);
	int CallGetBaud(unsigned char *data, int &count);

	// DeviceID setzen/lesen
	int CallSetDeviceIDBios(unsigned char *data, int &count);
	int CallGetDeviceIDBios(unsigned char *data, int &count);

	// Bootloader aktivieren
	int CallEnterBootloader(unsigned char *data, int &count);

	// Monitor Mode vom XPort
	int CallEnterXPortMonitor(unsigned char *data, int &count);
	int CallCommandXPort(unsigned char *data, int &count);
	int CallExitXPortMonitor(unsigned char *data, int &count);

	// Transparentsmodus fuer COM
	int CallOpenComServerMode(unsigned char *data, int &count);
	int CallCloseComServerMode(unsigned char *data, int &count);
	int CallOpenComServerModeOK(unsigned char *data, int &count);

	// Informationen zu einem Modultyp abfragen
	int CallGetInfo(unsigned char *data, int &count);
};

#endif
