/*!
    \file   df_timeboy_lists.h
    \author Bernd Ottmann
    \date   21.02.2003
    \brief  Deklarationen der Funktionen für die Arbeit mit Timeboylisten.
*/
#ifndef DF_TIMEBOY_LISTS_H
#define DF_TIMEBOY_LISTS_H

#include "df_defines.h"

#if (DF_API_VERSION >= 0x04010400)


/*!
    \brief Setzen des Listenbuffers fuer eine Timeboyliste.

    Die Listen werden intern kopiert.

    \param connectionId Kommunikationskanalnummer.
    \param groupNumber Gruppennummer der Liste
    \param number Listennummer.
    \param recordCount Anzahl der Zeilen, Datensaetze im Block.
    \param size Anzahl Byte im Block. Brechnet sich aus (Anzahl Datensaetzte    Datensatzlaenge).
    \param buffer Zeiger auf Bytearray, welches die Daten enthaelt.
    \param flags Angebbare Flags zur Liste. Z.B. Bit 1 ob Liste geloescht werden soll.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 04.01.04 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCMakeTimeboyList( int connectionId, int groupNumber, int number, int recordCount, int size, unsigned char *buffer, int flags, int *errorNumber );


/*!
    \brief Uebertragen der Listendaten zum Geraet.

    Es wird versucht, die zuvor mit DFCMakeTimeboyList() importierten Listendaten in
    das Geraet zu uebertragen.

    \param connectionId Kommunikationskanalnummer.
    \param deviceAddress Busnummer des anzusprechenden Geraetes.
    \param groupNumber Gruppennummer der Liste. (-1 bedeutet alle Gruppen)
    \param number Listennummer der zu uebertragenden Liste. (-1 uebertraegt alle importierten)
	\param unused_setTo0 Reservierter Parameter, aktuell mit 0 übergeben.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 04.01.04 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCLoadTimeboyList( int connectionId, int deviceAddress, int groupNumber, int number, int unused_setTo0, int *errorNumber );


/*!
    \brief Buffer fuer Timeboylisten loeschen.

    Loescht in der DLL den Buffer fuer Timeboylisten.

    \param connectionId Kommunikationskanalnummer.
    \param groupNumber Gruppennummer der Liste.
    \param number Listennummer (0 - 19). Bei Angabe von -1 werden alle geloescht.

    \return Keine.

    \version Ab DLL - Version 04.01.04 enthalten.
*/
DF_API_EXTERN void DF_API_CALL DFCClearTimeboyListBuffer( int connectionId, int groupNumber, int number );

#endif
#endif
