#include "../df_files.h"
#include "../../DatafoxLibraryIV.h"
#include <vector>
#include <boost/algorithm/string/split.hpp>

DF_API_EXTERN int DF_API_CALL DFCSetupLaden( int connectionId, int deviceAddress, char *filePath, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)	{
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%s", __func__, connectionId, deviceAddress,
        (filePath == NULL) ? "" : filePath);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (filePath == NULL) || strlen(filePath) < 5 || (errorNumber == NULL) )	{
            if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
            return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen()) {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    // Setupdaten uebertragen
    int iRes = pChannel->writeStudioFile(deviceAddress, filePath, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCDownload( int connectionId, int deviceAddress, char *filePath, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%s", __func__, connectionId, deviceAddress,
        (filePath == NULL) ? "" : filePath);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (filePath == NULL) || strlen(filePath) < 5 || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    // Setupdaten aus Geraet holen
    iRes = pChannel->readStudioFile(deviceAddress, filePath, errorNumber);

    if (!iRes && (*errorNumber != IV_ERROR_SUCCESS))
    {
        pChannel->resetWithError(*errorNumber);
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    // Setupdaten in Datei geschrieben
    pChannel->logEnd("\t%i\t(%i)", *errorNumber, 1);

    return 1;
}

DF_API_EXTERN int DF_API_CALL DFCUpload( int connectionId, int deviceAddress, char *filePath, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (filePath == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t(%i)", (filePath == NULL) ? "" : filePath, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes;
    char fileName[0x200];
    memset(fileName, 0, sizeof(fileName));
    strcpy(fileName, filePath);

    // Schnittstelle geoeffnet und CCom-Kanalklasse?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%i\t(%i)", fileName, *errorNumber, 0);
        return 0;
    }

    // Endung gibt funktion vor.
    // Nur ".dfl"
    size_t pos = strlen(fileName) - 4;

    if (fileName[pos] == '.' &&
        (((fileName[pos + 1] == 'd' || fileName[pos + 1] == 'D') &&
        (fileName[pos + 2] == 'f' || fileName[pos + 2] == 'F') &&
        (fileName[pos + 3] == 'l' || fileName[pos + 3] == 'L'))) )
    {
        // Textdaten uebertragen
        iRes = pChannel->writeTextData(deviceAddress, fileName);
        if (iRes == IV_ERROR_SUCCESS)
        { // Kein Fehler

            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if (iRes > IV_ERROR_SUCCESS)
        { // Fehler bei Durchfuehrung des Befehls

            // Modulfehler auswerten.
            *errorNumber = IV_ERROR_PROCESSING;
            iRes = 0;
        }
        else
        { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

            *errorNumber = -iRes;
            iRes = 0;
        }
    } 
    else if (fileName[pos] == '.' &&
        (((fileName[pos + 1] == 'd' || fileName[pos + 1] == 'D') &&
        (fileName[pos + 2] == 'f' || fileName[pos + 2] == 'F') &&
        (fileName[pos + 3] == 'k' || fileName[pos + 3] == 'K'))) )
    {
        // Touchdaten uebertragen
        iRes = pChannel->writeTouchKeyData(deviceAddress, fileName);
        if (iRes == IV_ERROR_SUCCESS)
        { // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if (iRes > IV_ERROR_SUCCESS)
        { // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ((iRes >> 8) & 0xFF) 
            {
                if (((iRes >> 8) & 0xFF) == 2) 
                {
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                } 
                else 
                {
                    *errorNumber = IV_ERROR_PROCESSING;
                }
            }
            else
            {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            iRes = 0;
        }
        else
        { // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    } 
    else if (fileName[pos] == '.' &&
        (((fileName[pos + 1] == 'd' || fileName[pos + 1] == 'D') &&
        (fileName[pos + 2] == 'f' || fileName[pos + 2] == 'F') &&
        (fileName[pos + 3] == 'c' || fileName[pos + 3] == 'C'))) )
    {
        // Farbdaten uebertragen
        iRes = pChannel->writeDisplayColors(deviceAddress, fileName);
        if (iRes == IV_ERROR_SUCCESS)
        { // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if (iRes > IV_ERROR_SUCCESS)
        { // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ((iRes >> 8) & 0xFF) 
            {
                if (((iRes >> 8) & 0xFF) == 2) 
                {
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                } 
                else 
                {
                    *errorNumber = IV_ERROR_PROCESSING;
                }
            }
            else
            {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            iRes = 0;
        }
        else
        { // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    } 
    else 
    {
        // Daten uebertragen
        iRes = pChannel->writeFileDevice(deviceAddress, fileName, errorNumber);
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%i\t(%i)", fileName, *errorNumber, iRes);

    if ((iRes == 0) && (*errorNumber == IV_ERROR_DEVICEFILE_MATCH))
    {
        // Zusatzinfo in Logdatei schreiben
        int count = 0;
        std::string errorText( pChannel->GetFileDeviceError() );
        std::vector<std::string> result;
        boost::split( result, errorText, [](char c) { return c == '\n'; } );
        for ( std::string &token : result )
        {
            pChannel->logBegin("ERROR.%s\t%i\t%i\t%s", __func__, connectionId, ++count, token.data());
            pChannel->logEnd("");
        }

        // Hardwareinfo als zusaetzliche Information rausgeben
        errorText = pChannel->GetFileDeviceInfo();
        if ( errorText.size() )
        {
            count = 0;
            result.clear();
            boost::split( result, errorText, [](char c) { return c == '\n'; } );
            for ( std::string &token : result )
            {
                pChannel->logBegin("INFO.%s\t%i\t%i\t%s", __func__, connectionId, ++count, token.data() );
                pChannel->logEnd("");
            }
        }
    }

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCUploadModule( int connectionId, int deviceAddress, int fileType, char *filePath, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, fileType);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) || (fileType != 0) ||
        (filePath == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t(%i)", (filePath == NULL) ? "" : filePath, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes;
    // Schnittstelle geoeffnet und CCom-Kanalklasse?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%i\t(%i)", filePath, *errorNumber, 0);
        return 0;
    }

    // Dateityp ermitteln
    struct stat st;
    if (stat(filePath, &st) == -1)
    {
        *errorNumber = IV_ERROR_FILE_OPEN;
        return 0;
    }
    // Nur ".up3" && ".up4" && ".up5" gueltig.
    size_t pos = strlen(filePath) - 4;
    if (
#ifdef _MSC_VER
        !((st.st_mode & _S_IFMT) == _S_IFREG) 
#else
        !((st.st_mode & S_IFMT) == S_IFREG) 
#endif
        || filePath[pos] != '.' || 
        !(filePath[pos + 1] == 'u' || filePath[pos + 1] == 'U') ||
        !(filePath[pos + 2] == 'p' || filePath[pos + 2] == 'P') ||
        !(filePath[pos + 3] == '3' || filePath[pos + 3] == '4' || filePath[pos + 3] == '5')) {

            *errorNumber = IV_ERROR_FILE_EXTENSION;
            return 0;
    }

    // Daten uebertragen
    iRes = pChannel->writeModule(deviceAddress, fileType, filePath, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%i\t(%i)", filePath, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCModifyStudioFile( char *sourceFilePath, char *destFilePath, char *xmlData, int flags, int *errorNumber )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s", __func__);

    // zulaessige Bereiche pruefen
    if ((sourceFilePath == NULL) || (destFilePath == NULL) || (xmlData == NULL) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%s\t%s\t%i\t(%i)", (sourceFilePath != NULL) ? sourceFilePath : "", (destFilePath != NULL) ? destFilePath : "", (xmlData != NULL) ? xmlData : "", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    int iRes = pChannel->modifyStudioFile(sourceFilePath, destFilePath, xmlData, flags, errorNumber);

    pChannel->logEnd("\t%s\t%s\t%s\t%i\t(%i)", sourceFilePath, destFilePath, xmlData, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCFileUpload( int connectionId, int deviceAddress, int fileType, char *filePath, int flags, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, fileType );

    // zulaessige Bereiche pruefen
    if (!((deviceAddress >= 1 && deviceAddress <= 250) || deviceAddress == 254) 
        || filePath == NULL || errorNumber == NULL 
        || fileType < -1 || fileType > 4 )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", (filePath == NULL) ? "" : filePath, flags, IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;

    int iRes;
    char _fileName[0x200];
    memset( _fileName, 0, sizeof( _fileName ) );
    strcpy( _fileName, filePath );

    // Schnittstelle geoeffnet und CCom-Kanalklasse?
    if ( !pChannel->isOpen() )
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", filePath, flags, *errorNumber, 0 );
        return 0;
    }

    // Endung gibt funktion vor.
    // Nur ".dfl"
    size_t pos = strlen( _fileName ) - 4;

    if ( fileType == 0 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'l' || _fileName[pos + 3] == 'L')) )
    {
        // Textdaten uebertragen
        iRes = pChannel->writeTextData( deviceAddress, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            *errorNumber = IV_ERROR_PROCESSING;
            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    }
    else if ( fileType == 1 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'k' || _fileName[pos + 3] == 'K')) )
    {
        // Touchdaten uebertragen
        iRes = pChannel->writeTouchKeyData( deviceAddress, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                else 
                    *errorNumber = IV_ERROR_PROCESSING;
            }
            else
            {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    }
    else if ( fileType == 2 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'c' || _fileName[pos + 3] == 'C')) )
    {
        // Farbdaten uebertragen
        iRes = pChannel->writeDisplayColors( deviceAddress, filePath );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                else 
                    *errorNumber = IV_ERROR_PROCESSING;
            }
            else
            {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }

            iRes = 0;

        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    }
    else if ( fileType == 3 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == '0' || _fileName[pos + 3] == '0')) )
    {
        // WLAN-Konfiguration uebertragen
        iRes = pChannel->writeWlanConfigData( deviceAddress, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                else 
                    *errorNumber = IV_ERROR_PROCESSING;
            }
            else
            {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    }
    else if ( (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == '1' || _fileName[pos + 3] == '1')) )
    {
        // Hardwarekonfiguration uebertragen
        iRes = pChannel->writeHardwareConfigData( deviceAddress, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                else 
                    *errorNumber = IV_ERROR_PROCESSING;
            }
            else
            {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    }
    else if ( fileType == 4 )
    {
        // Datei beliebiger Daten uebertragen
        iRes = pChannel->writeFileData( deviceAddress, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                else 
                    *errorNumber = IV_ERROR_PROCESSING;
            } else {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    }
    else
    {
        // Angegebener Type oder Dateiendung ist unbekannt.
        *errorNumber = IV_ERROR_FILE_EXTENSION;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", filePath, flags, *errorNumber, iRes );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCFileDownload( int connectionId, int deviceAddress, int fileType, char *filePath, int flags, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, fileType );

    // zulaessige Bereiche pruefen
    if (!((deviceAddress >= 1 && deviceAddress <= 250) || deviceAddress == 254) 
        || filePath == NULL || errorNumber == NULL 
        || fileType < -1  || fileType > 4 )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", (filePath == NULL) ? "" : filePath, flags, IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;

    int iRes;
    char _fileName[0x200];
    memset(_fileName, 0, sizeof(_fileName));
    strcpy(_fileName, filePath);

    // Schnittstelle geoeffnet und CCom-Kanalklasse?
    if ( !pChannel->isOpen() )
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", filePath, flags, *errorNumber, 0 );
        return 0;
    }

    // Endung kann funktion vorgeben.
    size_t pos = strlen( _fileName ) - 4;

    if ( fileType == 0 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'l' || _fileName[pos + 3] == 'L')) )
    {
        // Textdaten lesen
        *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
        iRes = 0;
    }
    else if ( fileType == 1 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'k' || _fileName[pos + 3] == 'K')) )
    {
        // Touchdaten uebertragen
        *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
        iRes = 0;
    }
    else if ( fileType == 2 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'c' || _fileName[pos + 3] == 'C')) )
    {
        // Farbdaten uebertragen
        *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
        iRes = 0;
    }
    else if ( fileType == 3 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == '0' || _fileName[pos + 3] == '0')) )
    {
        // WLAN-Konfiguration uebertragen
        iRes = pChannel->readWlanConfigData( deviceAddress, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                else 
                    *errorNumber = IV_ERROR_PROCESSING;
            }
            else
            {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            
            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    }
    else if ( (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == '1' || _fileName[pos + 3] == '1')) )
    {
        // Hardware-Konfiguration Daten uebertragen
        iRes = pChannel->readHardwareConfigData( deviceAddress, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                else 
                    *errorNumber = IV_ERROR_PROCESSING;
            }
            else
            {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            
            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    }
    else if ( fileType == 4 )
    {
        // Datei mit beliebigen Daten uebertragen
        iRes = pChannel->readFileData( deviceAddress, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            *errorNumber = IV_ERROR_SUCCESS;
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                    *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
                else 
                    *errorNumber = IV_ERROR_PROCESSING;
            }
            else
            {
                // Modulfehler auswerten.
                *errorNumber = IV_ERROR_PROCESSING;
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            *errorNumber = -iRes;
            iRes = 0;
        }
    }
    else
    {
        // Angegebener Type oder Dateiendung ist unbekannt.
        *errorNumber = IV_ERROR_FILE_EXTENSION;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd("\t%s\t%i\t%i\t(%i)", filePath, flags, *errorNumber, iRes);

    return iRes;
}
