#include "../df_timeboy_lists.h"
#include "../../DatafoxLibraryIV.h"

DF_API_EXTERN void DF_API_CALL DFCClearTimeboyListBuffer( int connectionId, int groupNumber, int number )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, groupNumber, number);

    // zulaessige Bereiche pruefen
    if ((groupNumber < -1) || (groupNumber > 19) || (number < -1) || (number > 19))
    {
        pChannel->logEnd("");
        return;
    }

    pChannel->ClearTimeboyListBuffer(groupNumber, number);

    pChannel->logEnd("");
}

DF_API_EXTERN int DF_API_CALL DFCMakeTimeboyList( int connectionId, int groupNumber, int number, int recordCount, int size, unsigned char *buffer, int flags, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, groupNumber, number, recordCount, size);

    // zulaessige Bereiche pruefen
    if ((groupNumber < 0) || (groupNumber > 19) || (number < 0) || (number > 19) || (buffer == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t%i\t(%i)", flags, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes = pChannel->MakeTimeboyList(groupNumber, number, recordCount, buffer, size, flags, errorNumber);

    pChannel->logEnd("\t\t%i\t%i\t(%i)", flags, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCLoadTimeboyList( int connectionId, int deviceAddress, int groupNumber, int number, int unused_setTo0, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, groupNumber, number, unused_setTo0);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (groupNumber < -1) || (groupNumber > 19) ||
        (number < -1) || (number > 19) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    // Timeboy Listenuebertragung wird erst ab 04.01.xx unterstuetzt.
    unsigned char protocolVersion = pChannel->getProtocolVersion(deviceAddress);
    if (protocolVersion == 0)
    {
        *errorNumber = IV_ERROR_UNREACHABLE;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }
    else if (protocolVersion < 4)
    {
        *errorNumber = IV_ERROR_VERSION_FIRMWARE;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes;
    char version[32];
    memset(version, 0, sizeof(version));
    // Firmwareversion lesen
    if (!(iRes = pChannel->getVersionFirmware(deviceAddress, version, errorNumber)))
    {
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }
    /*
    // Firmwareversion pruefen
    if (!theApp.m_util.CheckVersionFirmwareZK(version)) {

    *piError = IV_ERROR_FUNCTION_UNAVAILABLE;
    pChannel->logEnd("\t%i\t(%i)", *piError, 0);
    return 0;
    }
    */

    // Listendaten wegschreiben.
    if (pChannel->logIsWriteList())
    {
        char fileNameBin[_MAX_PATH];
        char basePath[_MAX_PATH];
        int listID, listNum;
        time_t t;
        tm *ltm;

        t = time(NULL);
        ltm = localtime(&t);

        for (listID=0; listID < MAX_DATENSATZ_BESCHREIBUNG; listID++)
            for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++)
            {
                // Dateiname der Logdatei
                sprintf(fileNameBin, LOG_LIST_FILENAME_BIN, pChannel->logGetBasePath(basePath),
                    ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
                    ltm->tm_hour, ltm->tm_min, ltm->tm_sec, connectionId, (char)(0x30 + listID), listNum);

                // Daten in die Datei schreiben.
                pChannel->WriteListTimeboyToBinFile( listID, listNum, fileNameBin );
            }
    }

    iRes = pChannel->writeTimeboyListData(deviceAddress, groupNumber, number, unused_setTo0, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}
