/**
 * @file  AEBuffer.cpp
 * @brief Implementierung der Klasse CAEBuffer.
 */
//////////////////////////////////////////////////////////////////////

#ifdef DATAFOX_STUDIO_IV
#ifdef _AFXDLL
//#include "../../DatafoxStudioIV/application.h"
#else
#include "globalUI.h"
#endif
#else
#include "../../DatafoxLibraryIV/DatafoxLibraryIV.h"
#endif
#include "AEBuffer.h"
#include "../../DatafoxLibraryIV/libraryversion.h"
#include "AEErrorDef.h"
#ifndef _MSC_VER
#include <ctype.h>
#endif
#include <stack>
#include <algorithm>
#include <vector>

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CAEBuffer::CAEBuffer()
{
    // Member Initialisieren
    StartDatensatzB = 0;
    EndDatensatzB = 0;
    MaxDatB = 0;
    MaxListB = 0;
/*
    // Zeiger auf Listenspeicher initialisieren
    int i;
    for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

        m_ucListRecordSize[i] = 0;
        m_listSize[i] = 0;
        m_listSortCheck[i] = false;
        m_pucListMem[i] = NULL;
    }
*/
    // Datensatzgroesse fuer Zutrittslisten (Version 1) initialisieren.
    m_ucEntranceRecordSize[0] = ENTRANCE_RECORD_SIZE_0; // 0: Zeitzonen (22)
    m_ucEntranceRecordSize[1] = ENTRANCE_RECORD_SIZE_1; // 1: Zeitmodelle (35)
    m_ucEntranceRecordSize[2] = ENTRANCE_RECORD_SIZE_2; // 2: Personalstamm (19)
    m_ucEntranceRecordSize[3] = ENTRANCE_RECORD_SIZE_3; // 3: Feiertage (16)

    // Zeiger auf Zutrittslistenspeicher (Version 1) initialisieren
    for ( int i = 0; i < 4; ++i )
    {
        m_ulEntranceListSize[i] = 0;
        m_pucEntranceListMem[i] = NULL;
    }

    // Speicher des Dokuments initialisieren
    m_file.SetDefault();
    // Aktuelle Library-Version setzen.
    m_file.SetLibraryVersion( libraryVersion );
}

CAEBuffer::~CAEBuffer()
{
    reset();
}

//////////////////////////////////////////////////////////////////////
// Member
//////////////////////////////////////////////////////////////////////

void CAEBuffer::reset()
{
    // Member Initialisieren
    StartDatensatzB = 0;
    EndDatensatzB = 0;
    MaxDatB = 0;
    MaxListB = 0;

    // Reservierten Listenspeicher freigeben
    //ClearListBuffer();

    // Reservierten Zutrittslistenspeicher (Version 1) freigeben
    ClearEntranceListBuffer();

    // Setupdatei bereinigen
    m_file.SetDefault();
}

void CAEBuffer::ChangeOrderLong(unsigned char *Buf)
{
    unsigned char h;

    h = Buf[0];
    Buf[0] = Buf[3];
    Buf[3] = h;

    h = Buf[1];
    Buf[1] = Buf[2];
    Buf[2] = h;
}

void CAEBuffer::ULongToBinMSBFirst(unsigned char *Buf,unsigned int Val)
{
    Buf[3] = (unsigned char)(Val&0xFF);
    Val >>= 8;
    Buf[2] = (unsigned char)(Val&0xFF);
    Val >>= 8;
    Buf[1] = (unsigned char)(Val&0xFF);
    Val >>= 8;
    Buf[0] = (unsigned char)(Val&0xFF);
}

void CAEBuffer::ChangeOrderInt(unsigned char *Buf)
{
    unsigned char h;
    h = Buf[0];
    Buf[0] = Buf[1];
    Buf[1] = h;
}
/*
void CAEBuffer::ClearListBuffer()
{
    int i;
    for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

        m_listSize[i] = 0;
        if (m_pucListMem[i]) {

            delete[] m_pucListMem[i];
            m_pucListMem[i] = NULL;
        }
    }
}

int CAEBuffer::MakeListe(int ListNum, int ListCnt, int ListSize, unsigned char *Buf)
{
    int i;

    // Zuvor reservierten Speicher freigeben
    if (m_pucListMem[ListNum]) {

        delete[] m_pucListMem[ListNum];
        m_pucListMem[ListNum] = NULL;
    }

    // Speicher reservieren
    m_pucListMem[ListNum] = new unsigned char[ListSize + sizeof(unsigned int)];
    
    if (m_pucListMem[ListNum] == NULL) {

        return 0;
    }

    // Pointer in Tabelle der Listen-Pointer eintragen
//	*(unsigned int*)&m_pucListMem[ListNum * sizeof(unsigned int)] = m_ulListSize + m_ulListStart;
//	ChangeOrderLong((unsigned char*)&m_pucListMem[ListNum * sizeof(unsigned int)]);

    // Start fuer Listendaten berechnen
//	pDat = m_ulListSize + sizeof(unsigned int);	// Platz fuer Anzahl Zeilen freihalten

    // Datenueberlauf auffangen
/*	if ((pDat + ListSize) > sizeof(m_pucListMem)) {
        // Liste loeschen
        *(unsigned int*)&m_pucListMem[ListNum * sizeof(unsigned int)] = 0;
        return 0;
    }
*	
    // Listendaten kopieren
    for (i=0; i < ListSize; i++) {

        m_pucListMem[ListNum][sizeof(unsigned int) + i] = Buf[i];
    }

    // Anzahl Listenzeilen eintragen
    *(unsigned int*)&m_pucListMem[ListNum][0] = ListCnt;
    //ChangeOrderLong(&m_pucListMem[ListNum][0]);
    
    // Listen-Pointer auf erstes freies Byte im Listenspeicher setzen
    //m_ulListSize = pDat + ListSize;

    // Liste als unsortiert markieren
    m_listSortCheck[ListNum] = false;

    // Listengrosse fuer weitere Verarbeitung halten
    m_listSize[ListNum] = sizeof(unsigned int) + ListSize;

    return 1;
}
*/
bool CAEBuffer::comparing(char *left, char *right)
{
    return (strcmp(left, right) < 0);
}

void CAEBuffer::SortList(unsigned char *pucMem, unsigned char ucRecordSize, unsigned char ucOffs)
{
    std::vector<char*> valPtr;
    // Vector mit Zeigern auf die zu Vergleichenden Werte fuellen.
    unsigned int count = *(unsigned int*)pucMem, i;
    valPtr.resize(count);
    for (i = 0; i < count; i++)
        valPtr[i] = (char*)&pucMem[sizeof(unsigned int) + ucOffs + (i * ucRecordSize)];

    // Sortierung durchfuehren.
    stable_sort(valPtr.begin(), valPtr.end(), comparing);

    unsigned char *mem = new unsigned char[sizeof(unsigned int) + count * ucRecordSize];
    
    *(unsigned int*)mem = count;
    for (i = 0; i < count; i++)
        memcpy(&mem[sizeof(unsigned int) + i * ucRecordSize], valPtr.at(i) - ucOffs, ucRecordSize);

    memcpy(pucMem, mem, sizeof(unsigned int) + count * ucRecordSize);
    delete[] mem;
}

void CAEBuffer::QuicksortList(unsigned char *pucMem, unsigned char ucRecordSize, unsigned char ucOffs, int lLeftRecordNum, int lRightRecordNum)
{
    int lHelp1;
    std::stack<int> sf;

    for (;;)
    {
        while (lRightRecordNum > lLeftRecordNum)
        {
            lHelp1 = QuicksortListPartition(pucMem, ucRecordSize, ucOffs, lLeftRecordNum, lRightRecordNum);
            if ((lHelp1 - lLeftRecordNum) > (lRightRecordNum - lHelp1))
            {
                sf.push(lLeftRecordNum);
                sf.push(lHelp1 - 1);
                lLeftRecordNum = lHelp1 + 1;
            }
            else
            {
                sf.push(lHelp1 + 1);
                sf.push(lRightRecordNum);
                lRightRecordNum = lHelp1 - 1;
            }
        }
        if (sf.empty()) break;
        lRightRecordNum = sf.top(); sf.pop();
        lLeftRecordNum = sf.top(); sf.pop();
    }
}

int CAEBuffer::QuicksortListPartition(unsigned char *pucMem, unsigned char ucRecordSize, unsigned char ucOffs, int lLeftRecordNum, int lRightRecordNum)
{
    int lHelp1, lHelp2;

    lHelp1 = lLeftRecordNum - 1;
    lHelp2 = lRightRecordNum;

    for (;;) {

        while (strcmp((const char*)&pucMem[++lHelp1 * ucRecordSize + sizeof(unsigned int) + ucOffs], 
            (const char*)&pucMem[lRightRecordNum * ucRecordSize + sizeof(unsigned int) + ucOffs]) < 0);

        while (strcmp((const char*)&pucMem[--lHelp2 * ucRecordSize + sizeof(unsigned int) + ucOffs], 
            (const char*)&pucMem[lRightRecordNum * ucRecordSize + sizeof(unsigned int) + ucOffs]) > 0);

        if (lHelp1 >= lHelp2) {

            break;
        }

        if (strcmp((const char*)&pucMem[lHelp1 * ucRecordSize + sizeof(unsigned int) + ucOffs], 
            (const char*)&pucMem[lHelp2 * ucRecordSize + sizeof(unsigned int) + ucOffs]) != 0) {

                QuicksortListSwap(pucMem, ucRecordSize, lHelp1, lHelp2);
        }
    }

    if (strcmp((const char*)&pucMem[lHelp1 * ucRecordSize + sizeof(unsigned int) + ucOffs], 
        (const char*)&pucMem[lRightRecordNum * ucRecordSize + sizeof(unsigned int) + ucOffs]) != 0) {

            QuicksortListSwap(pucMem, ucRecordSize, lHelp1, lRightRecordNum);
    }

    return lHelp1;
}

void CAEBuffer::QuicksortListSwap(unsigned char *pucMem, unsigned char ucRecordSize, int lLeftRecordNum, int lRightRecordNum)
{
    int i;
    unsigned char ucTmp;

    for (i=0; i < ucRecordSize; i++) {

        ucTmp = pucMem[lLeftRecordNum * ucRecordSize + sizeof(unsigned int) + i];
        pucMem[lLeftRecordNum * ucRecordSize + sizeof(unsigned int) + i] = pucMem[lRightRecordNum * ucRecordSize + sizeof(unsigned int) + i];
        pucMem[lRightRecordNum * ucRecordSize + sizeof(unsigned int) + i] = ucTmp;
    }
}

void CAEBuffer::ClearEntranceListBuffer(int iListNum)
{
    int i, j;
    if (iListNum == -1) { // Alle loeschen

        for (i=0; i < 4; i++) {
        
            m_ulEntranceListSize[i] = 0;
            m_ulCompressedListSize[i] = 0;
        }
    
        for (i=0; i < 4; i++) {

            // Reservierten Speicher freigeben
            if (m_pucEntranceListMem[i]) {

                delete[] m_pucEntranceListMem[i];
                m_pucEntranceListMem[i] = NULL;
            }

            for(j=0; j < FLASH_SIZE_LIST; j++) {

                m_pucCompressedListMem[i][j] = 0;
            }
        }

    } else {

        if (iListNum > 3) return;
        m_ulEntranceListSize[iListNum] = 0;


        // Reservierten Speicher freigeben
        if (m_pucEntranceListMem[iListNum]) {

            delete[] m_pucEntranceListMem[iListNum];
            m_pucEntranceListMem[iListNum] = NULL;
        }

        for (j=0; j < FLASH_SIZE_LIST; j++) {
        
            m_pucCompressedListMem[iListNum][j] = 0;
        }

        m_ulCompressedListSize[iListNum] = 0;
    }

}

int CAEBuffer::MakeEntranceList(int ListNum, int ListCnt, int ListSize, unsigned char *Buf)
{
    int i;

    // ListNum ist:
    // 0: Zeitzonen     (22 Byte) zu komprimierten  (6 Byte) max.   16 DS moeglich
    // 1: Zeitmodelle   (35 Byte) zu komprimierten  (3 Byte) max.   32 DS moeglich
    // 2: Personalstamm (19 Byte) zu komprimierten (16 Byte) max. 5120 DS moeglich
    // 3: Feiertage     (16 Byte) zu komprimierten  (6 Byte) max.   64 DS moeglich
    
    // Zuvor reservierten Speicher freigeben
    if (m_pucEntranceListMem[ListNum]) {

        delete[] m_pucEntranceListMem[ListNum];
        m_pucEntranceListMem[ListNum] = NULL;
    }

    // Speicher reservieren
    m_pucEntranceListMem[ListNum] = new unsigned char[ListSize + sizeof(unsigned int)];

    if (m_pucEntranceListMem[ListNum] == NULL) {

        return 0;
    }

    // Rohe Listendaten kopieren
    for (i=0; i < ListSize; i++) {

        m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i] = Buf[i]; // Platz fuer Anzahl Zeilen freihalten
    }


    // Anzahl Listenzeilen eintragen
    *(unsigned int*)&m_pucEntranceListMem[ListNum][0] = ListCnt;
    ChangeOrderLong(&(m_pucEntranceListMem[ListNum][0]));
    
    // Listenspeichergroesse setzen
    m_ulEntranceListSize[ListNum] = sizeof(unsigned int) + ListSize;


    // ##########################################
    // Komprimieren und sortieren der Listendaten
    // ##########################################

    // Liste Zeitzone - Komprimieren
    if (ListNum == 0) {

        structZeitZone *pZeitZone;
        unsigned int i, j;
        unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_0;
        unsigned char pucRecord[ENTRANCE_RECORD_SIZE_0];

        // Anzahl Listenzeilen eintragen
        *(unsigned int*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
        ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

        m_ulCompressedListSize[ListNum] = sizeof(unsigned int);

        // Liste mit -zeitzone- Daten komprimieren
        for (i=0; i < (unsigned int)ListCnt; i++) {
            
            for (j=0; j < ucRecordSize; j++) {

                // Datensatz aufbereiten.
                if (m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize + j]) {
                    // Abzug fuer ASCII - Werte
                    pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize + j] - 0x30;
                } else {
                    pucRecord[j] = 0x00;
                }
            }

            pZeitZone = (structZeitZone*)&m_pucCompressedListMem[ListNum][sizeof(unsigned int) + i * sizeof(structZeitZone)];
            pZeitZone->day = (unsigned char)((1 << pucRecord[0]) | (1 << pucRecord[1]) | (1 << pucRecord[2]) | (1 << pucRecord[3]) | (1 << pucRecord[4]) | (1 << pucRecord[5]) | (1 << pucRecord[6]));
            pZeitZone->start = ((unsigned short)(pucRecord[8] * 10 + pucRecord[9]) << 8) | (pucRecord[11] * 10 + pucRecord[12]);
            ChangeOrderInt((unsigned char*)&pZeitZone->start);
            pZeitZone->ende = ((unsigned short)(pucRecord[14] * 10 + pucRecord[15]) << 8) | (pucRecord[17] * 10 + pucRecord[18]);
            ChangeOrderInt((unsigned char*)&pZeitZone->ende);
            pZeitZone->aktiv = pucRecord[20];

            // Strukturoffsets zu Listengroesse aufrechnen.
            m_ulCompressedListSize[ListNum] += sizeof(structZeitZone);
        }
    }

    // Liste Zeitmodelle - Komprimieren
    if (ListNum == 1) {

        structZeitModell *pZeitModell;
        unsigned int i, j;
        unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_1;
        unsigned char pucRecord[ENTRANCE_RECORD_SIZE_1];
        unsigned char ucOffs = 0;

        // Anzahl Listenzeilen eintragen
        *(unsigned int*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
        ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

        m_ulCompressedListSize[ListNum] = sizeof(unsigned int);

        // Liste mit -zeitzmodelle- Daten komprimieren
        for (i=0; i < (unsigned int)ListCnt; i++) {
            
            for (j=0; j < ucRecordSize; j++) {

                // Datensatz aufbereiten.
                if (m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize + j]) {
                    // Abzug fuer ASCII - Werte
                    pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize + j] - 0x30;
                } else {
                    pucRecord[j] = 0x00;
                }
            }

            pZeitModell = (structZeitModell*)&m_pucCompressedListMem[ListNum][sizeof(unsigned int) + i * sizeof(structZeitModell)];
            pZeitModell->zeitmodell = pucRecord[0] * 10 + pucRecord[1];
            ucOffs = 3;
            pZeitModell->zone = 0;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0001 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0002 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0004 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0008 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0010 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0020 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0040 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0080 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0100 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0200 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0400 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0800 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x1000 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x2000 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x4000 : 0x0000;
            ucOffs += 2;
            pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x8000 : 0x0000;

            ChangeOrderInt((unsigned char*)&pZeitModell->zone);

            // Strukturoffsets zu Listengroesse aufrechnen.
            m_ulCompressedListSize[ListNum] += sizeof(structZeitModell);
        }
    }

    // Liste Personalstamm - Komprimieren
    if (ListNum == 2) {
        
        // Personalstamm nach Personalnummer sortieren.
        //QuicksortEntranceRecord(ListNum, 0, 0, ListCnt - 1);
        QuicksortList(m_pucEntranceListMem[ListNum], m_ucEntranceRecordSize[ListNum], 0, 0, ListCnt - 1);


        structPersonalStamm *pPersonalStamm;
        unsigned int i, j;
        unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_2;
        unsigned char pucRecord[ENTRANCE_RECORD_SIZE_2];

        // Anzahl Listenzeilen eintragen
        *(unsigned int*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
        ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

        m_ulCompressedListSize[ListNum] = sizeof(unsigned int);
/*
        for (i=0; i < ListCnt; i++) {
        
            TRACE("%s\n", &m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize]);
        }
*/
        // Liste mit -personalstamm- Daten komprimieren
        for (i=0; i < (unsigned int)ListCnt; i++) {
            
            pPersonalStamm = (structPersonalStamm*)&m_pucCompressedListMem[ListNum][sizeof(unsigned int) + i * sizeof(structPersonalStamm)];
            
            for (j=0; j < ucRecordSize; j++) {

                if (j < 13) {

                    pPersonalStamm->pNr[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize + j];
                } 

                // Datensatz aufbereiten.
                if (m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize + j]) {
                    // Abzug fuer ASCII - Werte
                    pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize + j] - 0x30;
                } else {
                    pucRecord[j] = 0x00;
                }				
            }


            for (j=0; j < 13; j++) {
                
                if (!pPersonalStamm->pNr[j]) { // 0terminie suchen
                
                    break;
                }
            }
            for (; j < 14; j++) { // rest mit 0 belegen
                
                pPersonalStamm->pNr[j] = 0x00;
            }

            pPersonalStamm->zeitmodell = pucRecord[14] * 10 + pucRecord[15];			
            pPersonalStamm->prio = pucRecord[17];

            // Strukturoffsets zu Listengroesse aufrechnen.
            m_ulCompressedListSize[ListNum] += sizeof(structPersonalStamm);
        }
    }
    
    // Liste Feiertage - Komprimieren
    if (ListNum == 3) {

        structFeiertag *pFeiertag;
        unsigned int i, j;
        unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_3;
        unsigned char pucRecord[ENTRANCE_RECORD_SIZE_3];

        // Anzahl Listenzeilen eintragen
        *(unsigned int*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
        ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

        m_ulCompressedListSize[ListNum] = sizeof(unsigned int);

        // Liste mit -feiertage- Daten komprimieren
        for (i=0; i < (unsigned int)ListCnt; i++) {
            
            for (j=0; j < ucRecordSize; j++) {

                // Datensatz aufbereiten.
                if (m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize + j]) {
                    // Abzug fuer ASCII - Werte
                    pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(unsigned int) + i * ucRecordSize + j] - 0x30;
                } else {
                    pucRecord[j] = 0x00;
                }
            }

            pFeiertag = (structFeiertag*)&m_pucCompressedListMem[ListNum][sizeof(unsigned int) + i * sizeof(structFeiertag)];
            pFeiertag->zeitmodell = pucRecord[0] * 10 + pucRecord[1];
            // JJJJMMDD
            pFeiertag->datum = (unsigned int)(((pucRecord[9] * 1000 + pucRecord[10] * 100 + pucRecord[11] * 10 + pucRecord[12]) << 16) | ((pucRecord[6] * 10 + pucRecord[7]) << 8) | (pucRecord[3] * 10 + pucRecord[4]));
            ChangeOrderLong((unsigned char*)&pFeiertag->datum);
            pFeiertag->prio = pucRecord[14];

            // Strukturoffsets zu Listengroesse aufrechnen.
            m_ulCompressedListSize[ListNum] += sizeof(structFeiertag);
        }
    }
    
    return 1;
}

void CAEBuffer::CpyFromKonfigBuf(unsigned char *Des,unsigned char *Src,unsigned int Add,int Cnt)
{
    int i;
#ifdef _MSC_VER
//	ASSERT(Add >= FLASH_KONFIG_START);
#endif
    for (i=0; i < Cnt; i++) {
        
        Des[i] = Src[Add + i - FLASH_KONFIG_START];
    }
}

unsigned int CAEBuffer::ChangeOrderULong(unsigned int val)
{
    unsigned char buf[4];

    *(unsigned int *)buf = val;
    ChangeOrderLong(buf);
    return *(unsigned int *)buf;
} 

int CAEBuffer::GetTableCount(unsigned char ucList)
{
    if (ucList) {
        
        return MaxListB;
    } else {
        
        return MaxDatB;
    }
}

int CAEBuffer::GetTableRecord(unsigned char ucNum,unsigned char *szName, 
        unsigned char *pucFieldCount, unsigned char *pucVariable, unsigned char ucList)
{
    int i;

    if (ucList) {

        // Bei Listen kann es ntig sein auf die Zutrittslistendaten zu verzweigen.
        if ( ucNum >= 100 )
        {
            // Auf Zutrtittskontrolllisten verzweigen
            if (ucNum >= 100 && ucNum < 104) {
                // Version 1
                *pucVariable = 0;
                return GetTableRecordEntrance( ucNum, szName, pucFieldCount );
            } else if (ucNum >= 200 && ucNum < 208) {
                // Version 2 wird nicht durch diese Version untersttzt.
                return 0;
            }
        }

        if (ucNum >= MaxListB) {
            
            return 0;
        }
        
        for (i=0; i < (MAX_NAME - 1) && 
            ListenBeschreibung[ucNum].Name[i] != 0; i++) {

            szName[i] = ListenBeschreibung[ucNum].Name[i];
        }
        szName[i] = 0;

        *pucFieldCount = ListenBeschreibung[ucNum].Felder;
        *pucVariable = 0;
    } else {

        if (ucNum >= MaxDatB) {
            
            return 0;
        }

        for (i=0; i < (MAX_NAME - 1) && 
            DatensatzBeschreibung[ucNum].Name[i] != 0; i++) {

            szName[i] = DatensatzBeschreibung[ucNum].Name[i];
        }
        szName[i] = 0;

        *pucFieldCount = DatensatzBeschreibung[ucNum].Felder;
    }

    return 1;
}

int CAEBuffer::GetTableField(unsigned char ucTableNum, unsigned char ucFieldNum, 
        unsigned char *szName, unsigned char *pucType, 
        unsigned char *pucLength, unsigned char ucList)
{
    int i;

    if (ucList) {

        // Bei Listen kann es ntig sein auf die Zutrittslistendaten zu verzweigen.
        if ( ucTableNum >= 100 )
        {
            // Auf Zutrittskontrolllisten verzweigen
            if (ucTableNum >= 100 && ucTableNum < 104) {
                // Version 1
                return GetTableFieldEntrance( ucTableNum, ucFieldNum, szName, pucType, pucLength );
            } else if (ucTableNum >= 200 && ucTableNum < 208) {
                // Version 2 wird nicht durch diese Version untersttzt.
                return 0;
            }
        }

        if (ucTableNum >= MaxListB) {
            
            return 0;
        }

        if (ListenBeschreibung[ucTableNum].Felder <= ucFieldNum) {
            
            return 0;
        }

        for (i=0; i < (MAX_NAME - 1) && 
            ListenFeld[ucTableNum][ucFieldNum].Name[i] != 0; i++) {

            szName[i] = ListenFeld[ucTableNum][ucFieldNum].Name[i];
        }
        szName[i] = 0;

        *pucType   = ListenFeld[ucTableNum][ucFieldNum].Typ;
        *pucLength = ListenFeld[ucTableNum][ucFieldNum].Cnt;
    } else {

        if (ucTableNum >= MaxDatB) {
            
            return 0;
        }

        if (DatensatzBeschreibung[ucTableNum].Felder <= ucFieldNum) {
            
            return 0;
        }

        for (i=0; i < (MAX_NAME - 1) && 
            DatensatzFeld[ucTableNum][ucFieldNum].Name[i] != 0; i++) {

            szName[i] = DatensatzFeld[ucTableNum][ucFieldNum].Name[i];
        }
        szName[i] = 0;

        *pucType   = DatensatzFeld[ucTableNum][ucFieldNum].Typ;
        *pucLength = DatensatzFeld[ucTableNum][ucFieldNum].Cnt;
    }

    return 1;
}

int CAEBuffer::GetTableRecordEntrance(unsigned char ucNum,unsigned char *szName, 
        unsigned char *pucFieldCount)
{
    switch(ucNum - 100) {
    case 0:
        strcpy((char*)szName, "Zeitzonen");
        *pucFieldCount = 6;
        break;
    case 1:
        strcpy((char*)szName, "Zeitmodelle");
        *pucFieldCount = 16;
        break;
    case 2:
        strcpy((char*)szName, "Personalstamm");
        *pucFieldCount = 3;
        break;
    case 3:
        strcpy((char*)szName, "Feiertage");
        *pucFieldCount = 5;
        break;
    default:
        return 0;
    }

    return 1;
}

int CAEBuffer::GetTableFieldEntrance(unsigned char ucTableNum, unsigned char ucFieldNum, 
        unsigned char *szName, unsigned char *pucType, 
        unsigned char *pucLength)
{
    switch(ucTableNum - 100) {
    case 0: // Zeitzonen
        switch(ucFieldNum) {
        case 0:
            strcpy((char*)szName, "WTAG");
            *pucType   = 3;
            *pucLength = 8;
            break;
        case 1:
            strcpy((char*)szName, "Start (h)");
            *pucType   = 3;
            *pucLength = 3;
            break;
        case 2:
            strcpy((char*)szName, "Start (min)");
            *pucType   = 3;
            *pucLength = 3;
            break;
        case 3:
            strcpy((char*)szName, "Ende (h)");
            *pucType   = 3;
            *pucLength = 3;
            break;
        case 4:
            strcpy((char*)szName, "Ende (min)");
            *pucType   = 3;
            *pucLength = 3;
            break;
        case 5:
            strcpy((char*)szName, "Aktiv");
            *pucType   = 3;
            *pucLength = 2;
            break;
        }
        break;
    case 1: // Zeitmodelle
        switch(ucFieldNum) {
        case 0:
            strcpy((char*)szName, "BGR");
            *pucType   = 3;
            *pucLength = 3;
            break;
        case 1:	case 2:	case 3:	case 4:	case 5:	case 6:	case 7:	case 8:	
        case 9:	case 10: case 11: case 12: case 13: case 14: case 15: case 16:
            sprintf((char*)szName, "ZZ%i", ucFieldNum);
            *pucType   = 3;
            *pucLength = 3;
            break;
        default:
            return 0;
        }
        break;
    case 2: // Personalstamm
        switch(ucFieldNum) {
        case 0:
            strcpy((char*)szName, "Ausweis_Nr");
            *pucType   = 4;
            *pucLength = 14;
            break;
        case 1:
            strcpy((char*)szName, "BGR");
            *pucType   = 3;
            *pucLength = 3;
            break;
        case 2:
            strcpy((char*)szName, "Prio");
            *pucType   = 3;
            *pucLength = 2;
            break;
        default:
            return 0;
        }
        break;
    case 3: // Feiertage
        switch(ucFieldNum) {
        case 0:
            strcpy((char*)szName, "BGR");
            *pucType   = 3;
            *pucLength = 3;
            break;
        case 1:
            strcpy((char*)szName, "Tag");
            *pucType   = 3;
            *pucLength = 3;
            break;
        case 2:
            strcpy((char*)szName, "Monat");
            *pucType   = 3;
            *pucLength = 3;
            break;
        case 3:
            strcpy((char*)szName, "Jahr");
            *pucType   = 3;
            *pucLength = 5;
            break;
        case 4:
            strcpy((char*)szName, "Prio");
            *pucType   = 3;
            *pucLength = 2;
            break;
        default:
            return 0;
        }
        break;
    default:
        return 0;
    }

    return 1;
}

void CAEBuffer::DefaultIOModule(StructIOModule *pIOModule)
{
    int i;
    pIOModule->ucActiveU = 0x00;
    pIOModule->ucOnScreenU = 0x00;
    pIOModule->ucActiveI = 0x00;
    pIOModule->ucOnScreenI = 0x00;

    // Setzen der default Werte.
    for (i=0; i < 8; i++) {
        // Spannungskanaele
        sprintf((char*)pIOModule->szNameU[i], "U%i", i + 1);
        sprintf((char*)pIOModule->szUnitU[i], "V");
        pIOModule->fMinU[i] = 0;
        pIOModule->fMaxU[i] = 10;

        // Stomkanaele
        sprintf((char*)pIOModule->szNameI[i], "I%i", i + 1);
        sprintf((char*)pIOModule->szUnitI[i], "mA");
        pIOModule->fMinI[i] = 0;
        pIOModule->fMaxI[i] = 20;
    }
}

// Groesse der Setupdaten im Flash ermitteln
unsigned int CAEBuffer::GetFlashSetupSize()
{
    // ##############################################################
    // Pointer sind direkte Adressen die im AEIII+:
    //   1. Im Motorolla - Format vorliegen.
    //   2. Mit einem Basisoffset -FLASH_KONFIG_START- berechnet sind.

    // Pointer auf ListenBeschreibungen ermitteln
    unsigned int pLB = *(unsigned int*)(&m_file.m_eep[P_LIST_STRUCT]);

    //if (App->ByteOrder == BO_MOT) {
        
        ChangeOrderLong((unsigned char*)&pLB);
    //}
    pLB -= FLASH_KONFIG_START;

    // Pointerarray der Listenbeschreibungen durchschreiten
    int i;
    for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
    
        // Nullpointer suchen fuer Listenanzahl
        if (*(unsigned int*)&m_file.m_flash[pLB + i * sizeof(unsigned int)] == 0ul) {
        
            break;
        }
    }

    // Liste gefunden?
    if (i > 0) {
    
        // Pointer auf letzte definierte Liste
        pLB = *(unsigned int*)(&m_file.m_flash[pLB + (i - 1) * sizeof(unsigned int)]);

        //if (App->ByteOrder == BO_MOT) {
            
            ChangeOrderLong((unsigned char*)&pLB);
        //}
        pLB -= FLASH_KONFIG_START;

        StructDatensatzBeschreibung *pLBStruct = (StructDatensatzBeschreibung*)(&m_file.m_flash[pLB]);

        // Pointer auf Feldarray der Liste
        unsigned int pFeldArray = pLBStruct->P;
        //if (App->ByteOrder == BO_MOT) {
            
            ChangeOrderLong((unsigned char*)&pFeldArray);
        //}
        pFeldArray -= FLASH_KONFIG_START;

        // Felder vorhanden?
        if (pLBStruct->Felder) {
        
            // Pointer auf letztes Feld der Listenbeschreibung ermitteln.
            unsigned int pFeld = *(unsigned int*)&m_file.m_flash[pFeldArray + (pLBStruct->Felder - 1) * sizeof(unsigned int)];
            //if (App->ByteOrder == BO_MOT) {
                
                ChangeOrderLong((unsigned char*)&pFeld);
            //}
            pFeld -= FLASH_KONFIG_START;
            
            // Feldadresse + eigene Groesse bildet FlashSetupSize
            return pFeld + sizeof(StructDatensatzFeld);
        }


        // Keine Felder definiert!
        return pFeldArray;
    }
    
    // Keine Listenbeschreibung definiert!
    // Die Anzahl fuer Listenbeschreibungspointer muss mitberechnet werden,
    // um bei einer Uebertragung diese zu loeschen.
    return pLB + MAX_DATENSATZ_BESCHREIBUNG * sizeof(unsigned int);
}

unsigned int CAEBuffer::GetMDEEKFlashSize_0400()
{
    // Erweiterten Struktur lesen
    StructExtendedFlash extBegin = *(StructExtendedFlash*)m_file.m_flashExtended;
    
    // Erweiterte Struktur vorhanden?
    if (extBegin.check != EXTENDED_CHECK) {
    
        return 0;
    }

    // Strukturen und Eingabeketten fuer MDE lesen.
    if (extBegin.offsetEventChains == 0) {
    
        // Kein Eintrag bei neuem Dokument. Erst nach writeFlashImage
        // 1Byte Anzahl Eingabeketten fuer MDE
        return 1;
    
    }

    int Unterpunkte;
    unsigned int pEK;

    Unterpunkte = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetEventChains];
    //if (App->ByteOrder == BO_MOT) {

        ChangeOrderLong((unsigned char*)&Unterpunkte);
    //}
    if (Unterpunkte == 0) {
    
        // 1Byte Anzahl Eingabeketten fuer MDE
        return 1;
    }
    
    // Pointer der letzten Eingabekette ermitteln
    pEK = extBegin.offsetEventChains + sizeof(unsigned char) + (Unterpunkte - 1) * sizeof(unsigned int);	
    pEK = *(unsigned int*)(&m_file.m_flashExtended[pEK]);
    //if (App->ByteOrder == BO_MOT) {
        
        ChangeOrderLong((unsigned char*)&pEK);
    //}
    // 16kByte fuer FlashOffset + Basisadresse fuer Konfigdaten subtrahieren
    pEK -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

    // vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
    pEK += sizeof(StructBedien);
    StructDatensatzEinlesen *pDE = (StructDatensatzEinlesen*)&m_file.m_flashExtended[extBegin.offsetEventChains + pEK];

    // Pointer auf Feldarray der Eingabekette
    unsigned int pFeldArray = pDE->P;
    //if (App->ByteOrder == BO_MOT) {
        
        ChangeOrderLong((unsigned char*)&pFeldArray);
    //}
    pFeldArray -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

    // Felder vorhanden?
    if (pDE->Felder > 0) {

    
        // Pointer auf letztes Feld der Eingabekette ermitteln.
        unsigned int pFeld = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetEventChains + pFeldArray + (pDE->Felder - 1) * sizeof(unsigned int)];
        //if (App->ByteOrder == BO_MOT) {
            
            ChangeOrderLong((unsigned char*)&pFeld);
        //}
        pFeld -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
        
        //StructFeldEinlesen *pDF = (StructFeldEinlesen*)&m_file.m_flashExtended[extBegin.offsetEventChains + pFeld];
        
        // Feldadresse + eigene Groesse bildet FlashSetupSize
        return pFeld + sizeof(StructFeldEinlesen);
    }

    // Keine Eingabekettenfelder definiert!
    return pFeldArray;
}

unsigned int CAEBuffer::GetMDEEKFlashSize()
{
    // Erweiterten Struktur lesen
    StructExtendedFlash extBegin = *(StructExtendedFlash*)m_file.m_flashExtended;
    
    // Erweiterte Struktur vorhanden?
    if (extBegin.check != EXTENDED_CHECK) {
    
        return 0;
    }

    // Strukturen und Eingabeketten fuer MDE lesen.
    if (extBegin.offsetEventChains == 0) {
    
        // Kein Eintrag bei neuem Dokument. Erst nach writeFlashImage
        // 4Byte Anzahl Eingabeketten fuer MDE
        return 4;
    
    }

    int Unterpunkte;
    unsigned int pEK;

    Unterpunkte = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetEventChains];
    //if (App->ByteOrder == BO_MOT) {

        ChangeOrderLong((unsigned char*)&Unterpunkte);
    //}
    if (Unterpunkte == 0) {
    
        // 4Byte Anzahl Eingabeketten fuer MDE
        return 4;
    }
    
    // Pointer der letzten Eingabekette ermitteln
    pEK = extBegin.offsetEventChains + sizeof(unsigned int) + (Unterpunkte - 1) * sizeof(unsigned int);	
    pEK = *(unsigned int*)&m_file.m_flashExtended[pEK];
    //if (App->ByteOrder == BO_MOT) {
        
        ChangeOrderLong((unsigned char*)&pEK);
    //}
    // 16kByte fuer FlashOffset + Basisadresse fuer Konfigdaten subtrahieren
    pEK -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

    // vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
    pEK += sizeof(StructBedien);
    StructDatensatzEinlesen *pDE = (StructDatensatzEinlesen*)&m_file.m_flashExtended[extBegin.offsetEventChains + pEK];

    // Pointer auf Feldarray der Eingabekette
    unsigned int pFeldArray = pDE->P;
    //if (App->ByteOrder == BO_MOT) {
        
        ChangeOrderLong((unsigned char*)&pFeldArray);
    //}
    pFeldArray -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

    // Felder vorhanden?
    if (pDE->Felder > 0) {

    
        // Pointer auf letztes Feld der Eingabekette ermitteln.
        unsigned int pFeld = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetEventChains + pFeldArray + (pDE->Felder - 1) * sizeof(unsigned int)];
        //if (App->ByteOrder == BO_MOT) {
            
            ChangeOrderLong((unsigned char*)&pFeld);
        //}
        pFeld -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
        
        //StructFeldEinlesen *pDF = (StructFeldEinlesen*)&m_file.m_flashExtended[extBegin.offsetEventChains + pFeld];
        
        // Feldadresse + eigene Groesse bildet FlashSetupSize
        return pFeld + sizeof(StructFeldEinlesen);
    }

    // Keine Eingabekettenfelder definiert!
    return pFeldArray;
}

unsigned int CAEBuffer::GetEntranceEKFlashSize()
{
    // Erweiterten Struktur lesen
    StructExtendedFlash extBegin = *(StructExtendedFlash*)m_file.m_flashExtended;
    
    // Erweiterte Struktur vorhanden?
    if (extBegin.check != EXTENDED_CHECK) {
    
        return 0;
    }

    // Strukturen und Eingabeketten fuer Zutritt lesen.
    if (extBegin.offsetAccessControlEventChainsV1 == 0) {
    
        // Kein Eintrag bei neuem Dokument. Erst nach writeFlashImage
        // 4Byte Anzahl Eingabeketten fuer Zutritt
        return 4;
    
    }

    int Unterpunkte;
    unsigned int pEK;

    Unterpunkte = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV1];
    //if (App->ByteOrder == BO_MOT) {
        
        ChangeOrderLong((unsigned char*)&Unterpunkte);
    //}
    if (Unterpunkte == 0) {
    
        // 4Byte Anzahl Eingabeketten fuer Zutritt
        return 4;
    }
    
    // Pointer der letzten Eingabekette ermitteln
    pEK = extBegin.offsetAccessControlEventChainsV1 + sizeof(unsigned int) + (Unterpunkte - 1) * sizeof(unsigned int);	
    pEK = *(unsigned int*)(&m_file.m_flashExtended[pEK]);
    //if (App->ByteOrder == BO_MOT) {
        
        ChangeOrderLong((unsigned char*)&pEK);
    //}
    // 32kByte fuer FlashOffset + Basisadresse fuer Konfigdaten subtrahieren
    pEK -= (FLASH_SIZE_SU + EXTENDED_SIZE_SU + FLASH_KONFIG_START);

    // vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
    pEK += sizeof(StructBedien);
    StructDatensatzEinlesen *pDE = (StructDatensatzEinlesen*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV1 + pEK];

    // Pointer auf Feldarray der Eingabekette
    unsigned int pFeldArray = pDE->P;
    //if (App->ByteOrder == BO_MOT) {
        
        ChangeOrderLong((unsigned char*)&pFeldArray);
    //}
    pFeldArray -= (FLASH_SIZE_SU + EXTENDED_SIZE_SU + FLASH_KONFIG_START);

    // Felder vorhanden?
    if (pDE->Felder > 0) {

    
        // Pointer auf letztes Feld der Eingabekette ermitteln.
        unsigned int pFeld = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV1 + pFeldArray + (pDE->Felder - 1) * sizeof(unsigned int)];
        //if (App->ByteOrder == BO_MOT) {
            
            ChangeOrderLong((unsigned char*)&pFeld);
        //}
        pFeld -= (FLASH_SIZE_SU + EXTENDED_SIZE_SU + FLASH_KONFIG_START);
        
        //StructFeldEinlesen *pDF = (StructFeldEinlesen*)&m_file.m_flashExtended[extBegin.offsetEventChains + pFeld];
        
        // Feldadresse + eigene Groesse bildet FlashSetupSize
        return pFeld + sizeof(StructFeldEinlesen);
    }

    // Keine Eingabekettenfelder definiert!
    return pFeldArray;
}

unsigned int CAEBuffer::GetEntrance2EKFlashSize()
{
    // Erweiterten Struktur lesen
    StructExtendedFlash extBegin = *(StructExtendedFlash*)m_file.m_flashExtended;
    
    // Erweiterte Struktur vorhanden?
    if (extBegin.check != EXTENDED_CHECK) {
    
        return 0;
    }

    // Strukturen und Eingabeketten fuer Zutritt lesen.
    if (extBegin.offsetAccessControlEventChainsV2 == 0) {
    
        // Kein Eintrag bei neuem Dokument. Erst nach writeFlashImage
        // 4Byte Anzahl Eingabeketten fuer Zutritt
        return 4;
    
    }

    int Unterpunkte;
    unsigned int pEK;

    Unterpunkte = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2];
    //if (theApp.ByteOrder == BO_MOT) {
        
//		ChangeOrderLong((unsigned char*)&Unterpunkte);
    //}
    if (Unterpunkte == 0) {
    
        // 4Byte Anzahl Eingabeketten fuer Zutritt
        return 4;
    }
    
    // Pointer der letzten Eingabekette ermitteln
    pEK = extBegin.offsetAccessControlEventChainsV2 + sizeof(unsigned int) + (Unterpunkte - 1) * sizeof(unsigned int);	
    pEK = *(unsigned int*)(&m_file.m_flashExtended[pEK]);
    //if (theApp.ByteOrder == BO_MOT) {
        
//		ChangeOrderLong((unsigned char*)&pEK);
    //}
    // Basisadresse fuer Konfigdaten subtrahieren
    pEK -= FLASH_KONFIG_START;

    // vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
    pEK += sizeof(StructBedien);
    StructDatensatzEinlesen *pDE = (StructDatensatzEinlesen*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + pEK];

    // Pointer auf Feldarray der Eingabekette
    unsigned int pFeldArray = pDE->P;
    //if (theApp.ByteOrder == BO_MOT) {
        
        ChangeOrderLong((unsigned char*)&pFeldArray);
    //}
    pFeldArray -= FLASH_KONFIG_START;

    // Felder vorhanden?
    if (pDE->Felder > 0) {

    
        // Pointer auf letztes Feld der Eingabekette ermitteln.
        unsigned int pFeld = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + pFeldArray + (pDE->Felder - 1) * sizeof(unsigned int)];
        //if (theApp.ByteOrder == BO_MOT) {
            
            ChangeOrderLong((unsigned char*)&pFeld);
        //}
        pFeld -= FLASH_KONFIG_START;
        
        //StructFeldEinlesen *pDF = (StructFeldEinlesen*)&ExtendedMem[extBegin.offsetEventChains + pFeld];
        
        // Feldadresse + eigene Groesse bildet FlashSetupSize
        return pFeld + sizeof(StructFeldEinlesen);
    }

    // Keine Eingabekettenfelder definiert!
    return pFeldArray;
}

void CAEBuffer::PrepareEntrance2EK(unsigned int offs, unsigned char *buf)
{
    // Erweiterten Struktur lesen
    int num, numFeld, posPointer = 0, posData = 0, posFeldArray;
    StructExtendedFlash extBegin = *(StructExtendedFlash*)m_file.m_flashExtended;
    
    // Erweiterte Struktur vorhanden?
    if (extBegin.check != EXTENDED_CHECK) {
    
        return;
    }

    // Sind Keine Angaben zur ZK vorhanden?
    if (extBegin.offsetAccessControlEventChainsV2 == 0) {

        // Keine Unterpunkte
        *(unsigned int*)buf = 0;
        return;
    }

    int Unterpunkte;
    unsigned int pEK;

    Unterpunkte = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2];
    //if (theApp.ByteOrder == BO_MOT) {
        
//		ChangeOrderLong((unsigned char*)&pEK);
    //}
    if (Unterpunkte == 0) {
    
        // 4Byte Anzahl Eingabeketten fuer Zutritt
        return;
    }

    // Anzahl Eingabeketten eintragen
    *(unsigned int*)buf = Unterpunkte;
    posPointer = sizeof(unsigned int);
    posData    = sizeof(unsigned int) + Unterpunkte * sizeof(unsigned int);;
    
    // Alle Eingabeketten durchlaufen
    for (num=0; num < Unterpunkte; num++) {

        // Pointer der Eingabekette ermitteln
        pEK = extBegin.offsetAccessControlEventChainsV2 + sizeof(unsigned int) + num * sizeof(unsigned int);	
        pEK = *(unsigned int*)&m_file.m_flashExtended[pEK];
        //if (theApp.ByteOrder == BO_MOT) {
            
//			ChangeOrderLong((unsigned char*)&pEK);
        //}
        // Basisadresse fuer Konfigdaten subtrahieren
        pEK -= FLASH_KONFIG_START;

        // Zeiger auf Eingabekette uebernehmen (ohne Basisoffset: FLASH_KONFIG_START)
        *(unsigned int*)&buf[posPointer] = pEK;
        posPointer += sizeof(unsigned int);


        // vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
        // diese uebernehmen
        memcpy(&buf[posData], &m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + pEK], sizeof(StructBedien));
        posData += sizeof(StructBedien);
        // dann uebergehen
        pEK += sizeof(StructBedien);

        // Zeiger auf Struktur 
        StructDatensatzEinlesen *pDE = (StructDatensatzEinlesen*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + pEK];

        // Pointer auf Feldarray der Eingabekette
        unsigned int pFeldArray = pDE->P;
        //if (theApp.ByteOrder == BO_MOT) {
            
            ChangeOrderLong((unsigned char*)&pFeldArray);
        //}
        pFeldArray -= FLASH_KONFIG_START;

        // Zeiger auf Feldarray anpassen und Struktur uebernehmen
        ChangeOrderLong((unsigned char*)&pDE->P);
        pDE->P += offs;
        ChangeOrderLong((unsigned char*)&pDE->P);
        memcpy(&buf[posData], pDE, sizeof(StructDatensatzEinlesen));
        ChangeOrderLong((unsigned char*)&pDE->P);
        pDE->P -= offs;
        ChangeOrderLong((unsigned char*)&pDE->P);
        posData += sizeof(StructDatensatzEinlesen);

        posFeldArray = posData;
        posData += pDE->Felder * sizeof(unsigned int);
        // Felder abarbeiten
        for (numFeld=0; numFeld < pDE->Felder; numFeld++) {
        
            // Pointer auf Feld der Eingabekette ermitteln.
            unsigned int pFeld = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + pFeldArray + numFeld * sizeof(unsigned int)];
            //if (theApp.ByteOrder == BO_MOT) {
                
                ChangeOrderLong((unsigned char*)&pFeld);
            //}

            // Zeiger auf Feldarray anpassen und Struktur uebernehmen
            *(unsigned int*)&buf[posFeldArray] = pFeld + offs;
            ChangeOrderLong(&buf[posFeldArray]);
            posFeldArray += sizeof(unsigned int);

            pFeld -= FLASH_KONFIG_START;

            StructFeldEinlesen *pDF = (StructFeldEinlesen*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + pFeld];
            
            // Feldstruktur uebernehmen
            memcpy(&buf[posData], pDF, sizeof(StructFeldEinlesen));
            // Da die Strukturen der Felder hintereinander liegen ist zwischen den
            // Zeigern immer ein Unterschied von sizeof(StructFeldEinlesen)
            posData += sizeof(StructFeldEinlesen);
        }
    }
}

void CAEBuffer::AdjustExtendedMemPointer()
{
    // Erweiterten Struktur lesen
    unsigned int num;
    int numFeld;
    StructExtendedFlash extBegin = *(StructExtendedFlash*)m_file.m_flashExtended;
    
    // Erweiterte Struktur vorhanden?
    if (extBegin.check != EXTENDED_CHECK) {
    
        return;
    }
    
    unsigned int count;
    unsigned int rootOffset, childOffset, tmp, tmp2;
    ///////////////////////////////////////////////////////
    // MDE
    if (extBegin.offsetEventChains != 0) {

        // Zeiger der Eingabeketten fuer MDE anpassen
        count = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetEventChains];
        ChangeOrderLong((unsigned char*)&count);
        
        // Jede Eingabekette anspringen
        for (num=0; num < count; num++) {
            
            rootOffset = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetEventChains + (num + 1) * sizeof(unsigned int)];
            ChangeOrderLong((unsigned char*)&rootOffset);
            
            tmp = rootOffset;
            
            if (rootOffset > (FLASH_SIZE_SU + FLASH_KONFIG_START)) {
                
                rootOffset -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
                tmp -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
            } else {
                
                rootOffset += (FLASH_SIZE_SU + FLASH_KONFIG_START);
            }
            
            ChangeOrderLong((unsigned char*)&rootOffset);
            *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetEventChains + (num + 1) * sizeof(unsigned int)] = rootOffset;
            
            StructBedien bedien = *(StructBedien*)&m_file.m_flashExtended[extBegin.offsetEventChains + tmp];
            ChangeOrderLong((unsigned char*)&bedien.P);
            
            tmp2 = bedien.P;
            
            if (bedien.P > (FLASH_SIZE_SU + FLASH_KONFIG_START)) {
                
                bedien.P -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
                tmp2 -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
            } else {
                
                bedien.P += (FLASH_SIZE_SU + FLASH_KONFIG_START);
            }
            
            ChangeOrderLong((unsigned char*)&bedien.P);
            *(StructBedien*)&m_file.m_flashExtended[extBegin.offsetEventChains + tmp] = bedien;
            
            StructDatensatzEinlesen datensatzEinlesen = *(StructDatensatzEinlesen*)&m_file.m_flashExtended[extBegin.offsetEventChains + tmp2];
            ChangeOrderLong((unsigned char*)&datensatzEinlesen.P);
            
            tmp = datensatzEinlesen.P;
            
            if (datensatzEinlesen.P > (FLASH_SIZE_SU + FLASH_KONFIG_START)) {
                
                datensatzEinlesen.P -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
                tmp -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
            } else {
                
                datensatzEinlesen.P += (FLASH_SIZE_SU + FLASH_KONFIG_START);
            }
            
            ChangeOrderLong((unsigned char*)&datensatzEinlesen.P);
            *(StructDatensatzEinlesen*)&m_file.m_flashExtended[extBegin.offsetEventChains + tmp2] = datensatzEinlesen;
            
            for (numFeld=0; numFeld < datensatzEinlesen.Felder; numFeld++) {
                
                childOffset = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetEventChains + tmp + numFeld * sizeof(unsigned int)];
                ChangeOrderLong((unsigned char*)&childOffset);
                
                if (childOffset > (FLASH_SIZE_SU + FLASH_KONFIG_START)) {
                    
                    childOffset -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
                } else {
                    
                    childOffset += (FLASH_SIZE_SU + FLASH_KONFIG_START);
                }
                
                ChangeOrderLong((unsigned char*)&childOffset);
                *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetEventChains + tmp + numFeld * sizeof(unsigned int)] = childOffset;
            }
        }
    }

    ///////////////////////////////////////////////////////
    // Zutrittskontrolle Version 2

    /////////////////////////
    // Tabellen
    if (extBegin.offsetAccessControlTablesV2 != 0) {

        // Zeiger der Tabellen fuer ZK anpassen
        count = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlTablesV2];
        
        // Jede Tabelle anspringen
        for (num=0; num < count; num++) {
            
            rootOffset = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlTablesV2 + (num + 1) * sizeof(unsigned int)];
            // Die Zeiger auf die einzelnen Tabellen sind 
            tmp = rootOffset;
            
            if (rootOffset > FLASH_KONFIG_START) {
                
                rootOffset -= FLASH_KONFIG_START;
                tmp -= FLASH_KONFIG_START;
            } else {
                
                rootOffset += FLASH_KONFIG_START;
            }
            
            *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlTablesV2 + (num + 1) * sizeof(unsigned int)] = rootOffset;
            
            StructDatensatzBeschreibung datensatzBeschreibung = *(StructDatensatzBeschreibung*)&m_file.m_flashExtended[extBegin.offsetAccessControlTablesV2 + tmp];
            ChangeOrderLong((unsigned char*)&datensatzBeschreibung.P);
            
            tmp2 = datensatzBeschreibung.P;
            
            if (datensatzBeschreibung.P > FLASH_KONFIG_START) {
                
                datensatzBeschreibung.P -= FLASH_KONFIG_START;
                tmp2 -= FLASH_KONFIG_START;
            } else {
                
                datensatzBeschreibung.P += FLASH_KONFIG_START;
            }
            
            ChangeOrderLong((unsigned char*)&datensatzBeschreibung.P);
            *(StructDatensatzBeschreibung*)&m_file.m_flashExtended[extBegin.offsetAccessControlTablesV2 + tmp] = datensatzBeschreibung;
            
            // Zeiger auf die Felder abarbeiten
            for (numFeld=0; numFeld < datensatzBeschreibung.Felder; numFeld++) {
                
                childOffset = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlTablesV2 + tmp2 + numFeld * sizeof(unsigned int)];
                ChangeOrderLong((unsigned char*)&childOffset);
                
                if (childOffset > FLASH_KONFIG_START) {
                    
                    childOffset -= FLASH_KONFIG_START;
                } else {
                    
                    childOffset += FLASH_KONFIG_START;
                }
                
                ChangeOrderLong((unsigned char*)&childOffset);
                *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlTablesV2 + tmp2 + numFeld * sizeof(unsigned int)] = childOffset;
            }
        }
    }

    /////////////////////////
    // Eingabeketten
    if (extBegin.offsetAccessControlEventChainsV2 != 0) {

        // Zeiger der Eingabeketten fuer MDE anpassen
        count = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2];
        
        // Jede Eingabekette anspringen
        for (num=0; num < count; num++) {
            
            rootOffset = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + (num + 1) * sizeof(unsigned int)];
            
            tmp = rootOffset;
            
            if (rootOffset > FLASH_KONFIG_START) {
                
                rootOffset -= FLASH_KONFIG_START;
                tmp -= FLASH_KONFIG_START;
            } else {
                
                rootOffset += FLASH_KONFIG_START;
            }
            
            *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + (num + 1) * sizeof(unsigned int)] = rootOffset;
            
            StructBedien bedien = *(StructBedien*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + tmp];
            ChangeOrderLong((unsigned char*)&bedien.P);
            
            tmp2 = bedien.P;
            
            if (bedien.P > FLASH_KONFIG_START) {
                
                bedien.P -= FLASH_KONFIG_START;
                tmp2 -= FLASH_KONFIG_START;
            } else {
                
                bedien.P += FLASH_KONFIG_START;
            }
            
            ChangeOrderLong((unsigned char*)&bedien.P);
            *(StructBedien*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + tmp] = bedien;
            
            StructDatensatzEinlesen datensatzEinlesen = *(StructDatensatzEinlesen*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + tmp2];
            ChangeOrderLong((unsigned char*)&datensatzEinlesen.P);
            
            tmp = datensatzEinlesen.P;
            
            if (datensatzEinlesen.P > FLASH_KONFIG_START) {
                
                datensatzEinlesen.P -= FLASH_KONFIG_START;
                tmp -= FLASH_KONFIG_START;
            } else {
                
                datensatzEinlesen.P += FLASH_KONFIG_START;
            }
            
            ChangeOrderLong((unsigned char*)&datensatzEinlesen.P);
            *(StructDatensatzEinlesen*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + tmp2] = datensatzEinlesen;
            
            // Zeiger auf die Felder abarbeiten
            for (numFeld=0; numFeld < datensatzEinlesen.Felder; numFeld++) {
                
                childOffset = *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + tmp + numFeld * sizeof(unsigned int)];
                ChangeOrderLong((unsigned char*)&childOffset);
                
                if (childOffset > FLASH_KONFIG_START) {
                    
                    childOffset -= FLASH_KONFIG_START;
                } else {
                    
                    childOffset += FLASH_KONFIG_START;
                }
                
                ChangeOrderLong((unsigned char*)&childOffset);
                *(unsigned int*)&m_file.m_flashExtended[extBegin.offsetAccessControlEventChainsV2 + tmp + numFeld * sizeof(unsigned int)] = childOffset;
            }
        }
    }
}

