/**
 * @file  filedevicezip.h
 * @brief Schnittstelle fuer die Klasse FileDeviceZip.
 */
//////////////////////////////////////////////////////////////////////
#ifndef FILEDEVICEZIP_H
#define FILEDEVICEZIP_H

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

// Basisklasse einbinden
#include "filedevice.h"

// Fuer die Verwendung von StructHardwareInfo
#include "../Common/aes.h"

#ifdef _MSC_VER
#include <direct.h>
#include <io.h>

#include <time.h>
#include <string>
#include <set>
#include <functional>

#include <algorithm>
#include <iterator>
using std::back_inserter;

using namespace std;
typedef greater<std::string> sf;
typedef multiset<std::string, sf> ms;
typedef ms::iterator msIterator;

// Initialisierungsdatei des Zip-Archiv
#include "IniFile.h"
// MD5 Berechnung 
#include "../Common/md5.h"
// Routinen fuer die zip Unterstuetzung
#include "../zip_utils/zip.h"
using namespace Zip;

#else

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#endif

class FileDeviceZip : public FileDevice
{
public:
	FileDeviceZip();
	virtual ~FileDeviceZip();

	void reset();
	
	int CreateZipFromDirectory(char *path, char *fileZIP, char *defaultLng);

	int CheckZip(char *fileZIP);
	int SearchInfosDir(char *path, const StructHardwareInfo &hwInfo);
	int SearchInfosZip(char *fileZIP, const StructHardwareInfo &hwInfo);

	int UnzipDefaultDFL(char *fileZIP, char *fileDefaultDFL);

	void resetError();
	void setRecentZipError( const char *file, int line );

    void setErrorTextExtended( const std::string &text );
    void appendErrorTextExtended( const std::string &text );
    std::string errorTextExtended();

protected:
#ifdef _MSC_VER
	CIniFile m_iniFile;
public:
	char m_version[128];
	char m_timestamp[20];
#endif
public:
    std::string m_errorExtended;
};

#endif
